#include <utils.h>
#include "glext.h"
#include "models.h"
#include "jav_cb.h"

typedef struct {
  float        dist;            // bond length
  int          at1,at2;         // atom pointer to asu
  float        xat1[3],xat2[3]; // coordinates
  op_type      op1, op2;        // symmetry operators
  int          dupl;            // pointer to similar bond for duplicate
  float        bv;              // bond valence
               } bond_info_t;

typedef struct {
  int    z1, z2 ;     // atomic numbers
  int    val1,val2;   // valence 
  float  ro;          // bond valence parameter Ro
  float  b;           // bond valence parameter B
  int    info; 
  float  rmin,rmax;   // bond window 
               } bond_valence_t;

typedef struct {
  int atom;
  char *label;
  char *typeA;
  float bvs;
  int   cn;
               } bvs_list_type;

extern "C" void jav_bond_valence_sum_(int *km, int *n, float vet[], int cn[],
                                      float *gii, int *cod);
extern "C" int  jav_bvs_ndinfo_(int *wm, int *kat);
extern "C" void jav_bvs_dinfo_(int *wm, int *kat, int *nd, bond_info_t *dinfo);
extern "C" void jav_edit_bvpar_(int *nbvec, bond_valence_t *bvec, int *act);
extern "C" void jav_export_bv_ (char *filenam, size_t sz);

namespace GLextmr
{

void selected_atom_to_edit(GtkTreeSelection  *sele, gpointer data);
char *getStringSymm(int *koper, int nops=0);

    extern Model *ModelInfo;
    extern GtkWidget *GLWINDOW;


void on_close_bvs(GtkWidget *button, gpointer data)
{
  chiudi_dlg(button, data);
}

bool sort_by_atom ( bvs_list_type elem1, bvs_list_type elem2 )
{
   return (elem1.atom < elem2.atom);
}

bool sort_by_atomd ( bvs_list_type elem1, bvs_list_type elem2 )
{
   return (elem1.atom > elem2.atom);
}

bool sort_by_type ( bvs_list_type elem1, bvs_list_type elem2 )
{
   return (g_ascii_strcasecmp(elem1.typeA, elem2.typeA) < 0);
}

bool sort_by_typed ( bvs_list_type elem1, bvs_list_type elem2 )
{
   return (g_ascii_strcasecmp(elem1.typeA, elem2.typeA) > 0);
}

void sort_col1(GtkWidget *column, gpointer data)
{
    bool (*sort_by_col)(bvs_list_type, bvs_list_type);
    int ncol = GPOINTER_TO_INT(data);
    int ordin = get_int(column, "set_order");
    GtkWidget *treeview = get_widget(column, "father");
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
    GtkListStore *list_store = GTK_LIST_STORE(model);
    GtkTreeIter iter;

    int nbvs = n_children(treeview);
    std::vector<bvs_list_type> BVS;
    gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
    while (valid)
    {
       bvs_list_type bvs1;
       char *str1 = g_new0(char, 25);
       char *str2 = g_new0(char, 10);
       gtk_tree_model_get (model, &iter, 0, &(bvs1.atom),
                                         1, &str1,
                                         2, &str2,
                                         3, &(bvs1.bvs),
                                         4, &(bvs1.cn), -1);
       bvs1.label = g_strdup(str1);
       bvs1.typeA = g_strdup(str2);
       valid = gtk_tree_model_iter_next (model, &iter);
       BVS.push_back(bvs1);
    }

    if(ordin > 0)
       ordin = -1;
    else
       ordin = 1;
    g_object_set_data(G_OBJECT(column), "set_order", GINT_TO_POINTER(ordin));
    switch (ordin*(ncol+1))
    {
    case -1:
         sort_by_col = sort_by_atomd;
         break;
    case +1:
         sort_by_col = sort_by_atom;
         break;
    case -3:
         sort_by_col = sort_by_typed;
         break;
    case +3:
         sort_by_col = sort_by_type;
         break;
    }
    sort(BVS.begin(), BVS.end(), sort_by_col);
    gtk_list_store_clear(list_store);
    for(int i=0; i<nbvs; i++)
    {
        gtk_list_store_append (list_store, &iter);
        gtk_list_store_set (list_store, &iter, 0, BVS[i].atom,
             1, BVS[i].label, 2, BVS[i].typeA, 3, BVS[i].bvs,
             4, BVS[i].cn, -1);
    }
}

static void on_edit_row (GtkCellRendererText *cell, const gchar *path_string,
                          const gchar *new_text, gpointer data)
{
  GtkTreeModel *model = (GtkTreeModel *)data;
  GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
  GtkTreeIter iter;

  gint column = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (cell), "column"));
  gint gt = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (cell), "gtype"));

  gtk_tree_model_get_iter (model, &iter, path);
  switch (gt)
  {
  case G_TYPE_STRING:
       {
             gchar *old_text;
             gtk_tree_model_get (model, &iter, column, &old_text, -1);
             g_free (old_text);
             gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                            g_strdup(new_text), -1);
       } 
       break;
  case G_TYPE_FLOAT:
       {
             float val = (float) atof(new_text);
             gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                            val, -1);
       break;
       } 
  case G_TYPE_INT:
       {
             int val =  atoi(new_text);
             gtk_list_store_set (GTK_LIST_STORE (model), &iter, column,
                            val, -1);
       } 
       break;
  }
}

static void FillLista(GtkWidget *w, int n, bond_valence_t *bvec)
{
  GtkListStore *list_store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(w)));
  gtk_list_store_clear(GTK_LIST_STORE(list_store));
  GtkTreeIter iter;

  for(int i=0; i<n; i++)
  {
      gtk_list_store_append (list_store, &iter);
      gtk_list_store_set (list_store, &iter,
        0, get_nth_element(bvec[i].z1-1), 1, bvec[i].val1,
        2, get_nth_element(bvec[i].z2-1), 3, bvec[i].val2,
        4, bvec[i].ro, 5, bvec[i].b, 6, bvec[i].rmin, 7, bvec[i].rmax, -1);
  }
  gtk_widget_set_size_request(get_widget(GTK_WIDGET(w), "scrollwin"),
          -1, (int) MRmin(n*30, 500));
}

static void ReadLista(GtkWidget *w, bond_valence_t *bvec)
{
  GtkTreeModel *model;
  GtkTreeIter   iter;
  gboolean      iter_valid;

  model = gtk_tree_view_get_model (GTK_TREE_VIEW(w));

  int i = 0;
  for (iter_valid = gtk_tree_model_get_iter_first (model, &iter);
       iter_valid;
       iter_valid = gtk_tree_model_iter_next (model, &iter))

  {
      gtk_tree_model_get (model, &iter,
        1, &bvec[i].val1,
        3, &bvec[i].val2,
        4, &bvec[i].ro, 5, &bvec[i].b, 6, &bvec[i].rmin, 7, &bvec[i].rmax, -1);
      i++;
  }
}

void on_edit_bvs(GtkButton *but, GtkWidget *dlg)
{
  int whichModel = get_int(dlg, "whichModel");
  int nvec = ModelInfo[whichModel-1].numSpec, action = 1;
  int n = (nvec*(nvec+1))/2;
  bond_valence_t *bvec = g_new0(bond_valence_t, n);
  const char *titles[] = { "Atom1", "Ox.St.1", "Atom2", "Ox.St.2", "Ro", "B", "dmin", "dmax"};
  GType types[] = { G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INT, G_TYPE_FLOAT, G_TYPE_FLOAT, G_TYPE_FLOAT, G_TYPE_FLOAT };
  int  sizes[] = { 0, 0, 0, 0, 0, 0, 0, 0 };
  int  precs[] = { 0, 0, 0, 0, 3, 3, 3, 3 };
  bool editable[] = {false, true, false, true, true, true, true, true};
  int ncol = (int)(sizeof(sizes)/sizeof(sizes[0]));
  PangoFontDescription *fontlist = get_fontN(MONOSPACE_FONT);
  
  GtkWidget *dialog1 = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dialog1), "Edit bond valence parameters");
  gtk_container_set_border_width(GTK_CONTAINER(dialog1), 8);
  GtkWidget *vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dialog1));

  gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog1), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(dialog1), GTK_WINDOW(dlg));

  jav_edit_bvpar_(&n, bvec, &action);

  GtkWidget *list =
  new_selection(dialog1, vbox1, "bvedit", ncol, titles, sizes, types, precs,
                editable, G_CALLBACK(on_edit_row));
  widget_modify_font(list, fontlist);
  FillLista(list, n, bvec);
  gtk_widget_show(list);

  GtkWidget *button = OK_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dialog1), button, GTK_RESPONSE_OK);
   gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
          GTK_BUTTONBOX_SPREAD);
  button = DEFAULTS_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dialog1), button, GTK_RESPONSE_ACCEPT);
  button = CANCEL_button();
  gtk_dialog_add_action_widget(GTK_DIALOG(dialog1), button, GTK_RESPONSE_CANCEL);

  bool loop = true;
  while (loop)
  {
     int response = gtk_dialog_run(GTK_DIALOG(dialog1));
     loop = false;
     switch(response)
     {
     case GTK_RESPONSE_OK:
     {
        action = 2;
        ReadLista(list, bvec);
        jav_edit_bvpar_(&n, bvec, &action);
        float *bvs = g_new0(float, ModelInfo[whichModel-1].nats_au);
        int   *cn = g_new0(int, ModelInfo[whichModel-1].nats_au);
        float gii;
        int cod = 2;
        jav_bond_valence_sum_(&whichModel, &ModelInfo[whichModel-1].nats_au,
               bvs, cn, &gii, &cod);
//per Rosi: inserisci un update delle stringa GII = 
        ModelInfo[whichModel-1].FillBondValence(dlg, bvs, cn, gii);
        g_free(bvs);
        g_free(cn);
     }
     break;
     case GTK_RESPONSE_ACCEPT:
        action = 3;
        jav_edit_bvpar_(&n, bvec, &action);
        FillLista(list, n, bvec);
        loop = true;
     break;
     case GTK_RESPONSE_CANCEL:
        break;
     }
  }
  gtk_widget_destroy(dialog1);
}

void on_export_bvs(GtkButton *but, GtkWidget *dlg)
{
using namespace gtkext;
     std::string curdir = getPWD();
     std::string fn;
     int wm = get_int(dlg, "whichModel") - 1;
     if(ModelInfo[wm].created)
     {
        std::string nomemod =  g_path_get_basename(ModelInfo[wm].modname);
        size_t posi = nomemod.find_last_of('.');
        fn = nomemod.substr(0, posi);
     }
     else
        fn = "";

     fn =  dlg_select_file(dlg, fn, eSELECT_TO_WRITE, "*.*", false);
     if(!str_white(fn.c_str()))
     {
        char *filenam = g_strdup(fromUTF8(fn).c_str());
        size_t sz = strlen(filenam);
        jav_export_bv_ (filenam, sz);
        g_free(filenam);
     }
     dir_set_current(curdir.c_str());
}

GtkWidget *create_bvs_window (GtkWidget *window)
{
  GtkWidget *dialog1;
  GtkWidget *button;
  int lar, alt;
  const char *titles[] = { "Number", "Label", "Type", "BVS", "CN"};
  GType types[] = { G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_FLOAT, G_TYPE_INT};
  int  sizes[] = { 0, 0, 0, 0, 0};
  int precs[] = { 0, 0, 1, 5, 0};
  int sort_type[] = {0, -1, 2, -1, -1};
  const char *titles1[] = { "No.", "Label", "Type", "x", "y", "z", "Symm.Op.",
           "Distance", "Bond Valence"};
  GType types1[] = { G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_FLOAT,
                    G_TYPE_FLOAT, G_TYPE_FLOAT, G_TYPE_STRING, G_TYPE_FLOAT,
                    G_TYPE_FLOAT};
  int  sizes1[] = { 0, 0, 50, 0, 0, 0, 0, 0, 0};
  int precs1[] = { 0, 0, 1, 4, 4, 4, 0, 3, 3};


  PangoFontDescription *fontlist = get_fontN(MONOSPACE_FONT);
  get_metrics(fontlist, &lar, &alt);

  dialog1 = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dialog1), "Bond Valence Sum");
  gtk_container_set_border_width(GTK_CONTAINER(dialog1), 8);
  GtkWidget *vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dialog1));
  gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog1), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(dialog1), GTK_WINDOW(GLWINDOW));
  g_object_set_data(G_OBJECT(dialog1), "alt", GINT_TO_POINTER(alt));

  int inModify = GetActiveModel() - 1;
  g_object_set_data(G_OBJECT(dialog1), "whichModel", GINT_TO_POINTER(inModify+1));
  GtkWidget *parentbox = hbox_new (FALSE, 2);
  gtk_widget_show (parentbox);
  //gtk_box_pack_start(GTK_BOX(vbox1), parentbox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox1), parentbox, TRUE, TRUE, 0);

  int ncol = (int)(sizeof(sizes)/sizeof(sizes[0]));
  GtkWidget *list =
  new_selection(dialog1, parentbox, "bondvs", ncol, titles, sizes, types, precs);
  g_object_set_data(G_OBJECT(list), "father_b", parentbox);
  gtk_widget_set_name(parentbox, "bondvs");
  widget_modify_font(list, fontlist);
  for(int i=0; i<ncol; i++)
  {
      GtkTreeViewColumn *column = gtk_tree_view_get_column(GTK_TREE_VIEW(list), i);
      GtkWidget *label = gtk_tree_view_column_get_widget(column);
      if(label)
         widget_modify_font(label, fontlist);
      g_object_set_data(G_OBJECT(column), "father", list);
      if(sort_type[i] >= 0)
      {
         gtk_tree_view_column_set_clickable(column, TRUE);
         g_object_set_data(G_OBJECT(column), "set_order", GINT_TO_POINTER(-1));
         g_signal_connect(G_OBJECT(column), "clicked", G_CALLBACK(sort_col1),
                 GINT_TO_POINTER(sort_type[i]));
      }
  }
  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
  int hnd = g_signal_connect (G_OBJECT (selection), "changed",
                      G_CALLBACK (selected_atom_to_edit), dialog1);
  g_object_set_data(G_OBJECT(selection), "hnd", GINT_TO_POINTER(hnd));
  g_object_set_data(G_OBJECT(list), "hnd", GINT_TO_POINTER(hnd));

  GtkWidget *vbox2 = vbox_new (FALSE, 2);
  gtk_widget_show (vbox2);
  gtk_box_pack_start(GTK_BOX(parentbox), vbox2, FALSE, FALSE, 0);

  GtkWidget *label = LABEL_NEW(" ", NULL);
  gtk_box_pack_start(GTK_BOX(vbox2), label, FALSE, FALSE, 5);
  g_object_set_data(G_OBJECT(dialog1), "gii", label);
  gtk_widget_show (label);

  button = gtk_button_new_with_label("Edit bond valence parameters");
  gtk_box_pack_start(GTK_BOX(vbox2), button, FALSE, FALSE, 5);
  gtk_widget_show (button);
  (void) g_signal_connect(G_OBJECT (button), "clicked",
         G_CALLBACK(on_edit_bvs), dialog1);

  button = gtk_button_new_with_label("Export");
  gtk_box_pack_start(GTK_BOX(vbox2), button, FALSE, FALSE, 5);
  gtk_widget_show (button);
  (void) g_signal_connect(G_OBJECT (button), "clicked",
         G_CALLBACK(on_export_bvs), dialog1);

  GtkWidget *frame = FRAME_BOLD("Bonds of atom", MyBLUE, 1.1f);
  parentbox = vbox_new (FALSE, 2);
  gtk_container_add (GTK_CONTAINER (frame), parentbox);
  g_object_set_data(G_OBJECT(dialog1), "bondinfo_b", frame);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);

  int ncol1 = (int)(sizeof(sizes1)/sizeof(sizes1[0]));
  GtkWidget *list1 =
  new_selection(dialog1, parentbox, "bondinfo", ncol1, titles1, sizes1, types1, precs1);
  g_object_set_data(G_OBJECT(list1), "father_b", parentbox);
  gtk_widget_set_name(parentbox, "bondinfo");
  widget_modify_font(list1, fontlist);
  for(int i=0; i<ncol1; i++)
  {
      GtkTreeViewColumn *column = gtk_tree_view_get_column(GTK_TREE_VIEW(list1), i);
      GtkWidget *label = gtk_tree_view_column_get_widget(column);
      if(label)
         widget_modify_font(label, fontlist);
      g_object_set_data(G_OBJECT(column), "father", list1);
  }

  chiudi_dlg_data *st = g_new0(chiudi_dlg_data, 1);
  st->window = window;
  st->name = BondValenceSumWindow;

  button = CLOSE_button();
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), button, 1);
  (void) g_signal_connect(G_OBJECT (button), "clicked",
         G_CALLBACK(on_close_bvs), (gpointer) st);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
      GTK_BUTTONBOX_SPREAD);

  (void) g_signal_connect_swapped(G_OBJECT(dialog1), "delete_event",
        G_CALLBACK(gtk_button_clicked), G_OBJECT(button));
   (void) g_signal_connect (G_OBJECT (dialog1), "delete_event",
                      G_CALLBACK(true_cb), NULL);

  gtk_widget_show_all(dialog1);
  //gtk_container_add (GTK_CONTAINER (vbox1), frame);
  gtk_box_pack_start(GTK_BOX(vbox1), frame, TRUE, TRUE, 0);

  g_object_set_data(G_OBJECT(dialog1), "whichAtom", GINT_TO_POINTER(-1));
  return dialog1;
}

void Model::FillBondValence(GtkWidget *dlg, float *bvs, int *cn, float gii)
{
       int natoms = nats_au;

       std::string gii_str = "GII = "+fToStr(gii, 6);
       set_label_text(get_widget(dlg, "gii"), gii_str.c_str(), NULL, 1.2f, 0);
       GtkTreeView *list = GTK_TREE_VIEW(get_widget(dlg, "bondvs"));
       GtkTreeSelection *selection = gtk_tree_view_get_selection(list);
       GtkTreeIter iter;
       GtkListStore *list_store = GTK_LIST_STORE(gtk_tree_view_get_model(list));
       int hnd = get_int(GTK_WIDGET(list), "hnd");
       g_signal_handler_block(G_OBJECT(selection), hnd);
       gtk_list_store_clear(GTK_LIST_STORE(list_store));
       int alt = get_int(dlg, "alt");
       gtk_widget_set_size_request(get_widget(GTK_WIDGET(list), "scrollwin"),
                    -1, (int) MRmin(1.7f*(natoms+1)*alt, 20*alt));
       for(int i=0; i<natoms; i++)
       {
           char *label1 = g_strdup(atm_au[i].lab);
           char *label2;
           if(atm_au[i].color >= 0)
           {
              label2 = g_new0(char, LEN_SPEC+1);
              CopiaStringan( label2, LEN_SPEC+1, LabSpec[atm_au[i].color], LEN_SPEC);
           }
           else
              label2 = g_strdup("Q");
           gtk_list_store_append (list_store, &iter);
           float bvs1 = bvs[i];
           int cn1 = cn[i];
           gtk_list_store_set (list_store, &iter, 
              0, i+1, 1, g_strstrip(label1), 2, label2, 3, bvs1, 4, cn1, -1);
       }
       g_signal_handler_unblock(G_OBJECT(selection), hnd);
       select_row(GTK_WIDGET(list), 0, false);
}

void OpenBondValenceSumWindow(GtkWidget *window, GLWindow *gl, float *bvs, int *cn, float gii)
{
   GtkWidget *dlg;
   if(g_object_get_data(G_OBJECT(window), BondValenceSumWindow))
   {
      dlg = get_widget(window, BondValenceSumWindow);
      gdk_window_raise(gtk_widget_get_window(dlg));
   }
   else
   {
      dlg = create_bvs_window(window);
      g_object_set_data(G_OBJECT(window), BondValenceSumWindow, dlg);
      int inModify = GetActiveModel();
      g_object_set_data(G_OBJECT(dlg), "whichModel", GINT_TO_POINTER(inModify));
      //std::string gii_str = "GII = "+fToStr(gii, 6);
      //set_label_text(get_widget(dlg, "gii"), gii_str.c_str(), NULL, 1.2f, 0);
   }
   int whichModel = get_int(dlg, "whichModel");
   ModelInfo[whichModel-1].FillBondValence(dlg, bvs, cn, gii);
}

void on_bondvs(gpointer , gpointer data)
{
   GtkWidget *window = (GtkWidget *)data;
   int whichModel = GetActiveModel();
   if(whichModel == 0)
        return;
   if(!ModelInfo[whichModel-1].created)
        return;
   if(!ModelInfo[whichModel-1].nats_au)
        return;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(window), "glwindow");
   float *bvs = g_new0(float, ModelInfo[whichModel-1].nats_au);
   int   *cn = g_new0(int, ModelInfo[whichModel-1].nats_au);
   float gii;
   int cod = 1;
   jav_bond_valence_sum_(&whichModel, &ModelInfo[whichModel-1].nats_au, bvs,
            cn, &gii, &cod);
   OpenBondValenceSumWindow(window, gl, bvs, cn, gii);
   g_free(bvs);
   g_free(cn);
}


void Model::FillBondLengths(GtkWidget *but, int wm, int kat)
{
   int nrows = jav_bvs_ndinfo_(&wm, &kat);
   bond_info_t *dinfo = g_new0(bond_info_t, nrows);
   jav_bvs_dinfo_(&wm, &kat, &nrows, dinfo);

   gtk_frame_set_label(GTK_FRAME(but),
         g_strdup_printf("Bonds of atom %s", atm_au[kat-1].lab));
         //g_strdup_printf("Bond lengths of atom %s", atm[kat-1].lab));
   GtkWidget *dlg = gtk_widget_get_toplevel(but);
   GtkTreeView *list = GTK_TREE_VIEW(get_widget(dlg, "bondinfo"));
   GtkTreeIter iter;
   GtkListStore *list_store = GTK_LIST_STORE(gtk_tree_view_get_model(list));
   gtk_list_store_clear(GTK_LIST_STORE(list_store));
   int alt = get_int(dlg, "alt");
   gtk_widget_set_size_request(get_widget(GTK_WIDGET(list), "scrollwin"),
                    -1, 6*alt);
   //gtk_widget_set_size_request(get_widget(GTK_WIDGET(list), "scrollwin"),
                    //-1, (int) MRmin(1.7f*(nrows+1)*alt, 20*alt));
       for(int i=0; i<nrows; i++)
       {
           int n1 = dinfo[i].at2;
           char *label1 = g_strdup(atm_au[n1-1].lab);
           char *label2;
           if(atm_au[n1-1].color >= 0)
           {
              label2 = g_new0(char, LEN_SPEC+1);
              CopiaStringan( label2, LEN_SPEC+1, LabSpec[atm_au[n1-1].color], LEN_SPEC);
           }
           else
              label2 = g_strdup("Q");
           int opnum = dinfo[i].op2.op;
           int tra[3] = {dinfo[i].op2.tra[0],
                         dinfo[i].op2.tra[1], dinfo[i].op2.tra[2]};
           char *label3 = g_strdup(getStringSymm(&opnum));
           std::string labstr = g_strdup_printf("(%s)",g_strchomp(label3));
           if(tra[0] + tra[1] + tra[2])
              labstr.append(g_strdup_printf("+(%d,%d,%d)",tra[0], tra[1], tra[2]));
           label3 = g_strdup(labstr.c_str());

           gtk_list_store_append (list_store, &iter);
           gtk_list_store_set (list_store, &iter,
              0, n1, 1, g_strstrip(label1), 2, label2, 
              3, dinfo[i].xat2[0], 4, dinfo[i].xat2[1], 5, dinfo[i].xat2[2],
              6, label3, 7, dinfo[i].dist, 8, dinfo[i].bv, -1);
       }



   g_free(dinfo);
}

}   // end namespace
