#include <cstdio>
#include <cstring>
#include <utils.h>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <sir_application.h>
//#include <gtkdefs.h>
#define GL_GLEXT_PROTOTYPES 1
#include "glext.h"
#include "models.h"
#include "jav_cb.h"

extern "C" void jav_find_centroids_(int *kmodel);

namespace Centroids
{
    const double radC = 1;
    double radN;
    const int lenLab = 40;
class jav_centroid_type {
    public:
                 float xyz[3];
                 float xyzN[3];
                 int   nat;
                 int   *atoms;
                 bool  visible;
                 int   transparency;
                 float colore[4];
                 bool  showlab;
                 char  label[lenLab];
                 bool  selected;

void DrawSelected(float add)
{
     glPushMatrix();
     glTranslatef(xyzN[0], xyzN[1], xyzN[2]);
     GLextmr::disegnaSfera(radN+add, 100, false);
     glPopMatrix();
}

void Draw()
{
     float color1[4];
     memcpy(color1, colore, 4*sizeof(float));
     if(transparency)
     {
        glDepthMask (GL_FALSE);
        color1[3] = (float) (100 - transparency)/100.f;
     }
     else
        glDepthMask (GL_TRUE);

     glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color1);
     glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);
     glEnable (GL_BLEND);
     glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glPushMatrix();
     glTranslatef(xyzN[0], xyzN[1], xyzN[2]);
     GLextmr::disegnaSfera(radN, 100, false);
     glPopMatrix();
     glDisable (GL_BLEND);
     glDepthMask (GL_TRUE);
}

void DrawLabel(double delta[3],  PangoFontDescription *font_desc, float c[])
{
     double *model = g_new0(double, 16);
     double *proj = g_new0(double, 16);
     GLint view[4];
     GLdouble winx, winy, winz;
     GLdouble x, y, z;
     glGetDoublev(GL_MODELVIEW_MATRIX, model);
     glGetDoublev(GL_PROJECTION_MATRIX, proj);
     glGetIntegerv(GL_VIEWPORT, view);

     GLextmr::FromCoordToWin(xyzN[0],xyzN[1], xyzN[2],
                    model, proj, view, &winx, &winy, &winz);
     GLextmr::FromWinToCoord(winx+delta[0], winy+delta[1],
                winz-delta[2]/10, model, proj, view, &x, &y, &z);
     vet3 xy = {x, y, z};
     float ico[] = {c[0], c[1], c[2]};
     std::string str(label);

     GLextmr::DrawText(-1, font_desc, PANGO_ALIGN_CENTER, ico, xy,
         str.c_str(), NULL, true);
}
};  //end class definition

    std::vector <jav_centroid_type>AllCentr[MAX_NUM_MODELS];

}

namespace GLextmr
{
    extern Model *ModelInfo;
    extern GtkWidget *GLWINDOW;
    extern GLfloat sc_factor;
    extern GLint   npixels;

void close_centroids(GtkWidget *button, gpointer data)
{
    gtk_widget_destroy(gtk_widget_get_toplevel(button));
}

void find_centroids(GtkWidget *button, gpointer data)
{
   int whichModel = GetActiveModel();
   jav_find_centroids_(&whichModel);
}

void toggle_centvis(GtkToggleButton *but, gpointer data)
{
using namespace Centroids;
   if(!GLWINDOW)
      return;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
   int whichModel = GetActiveModel() - 1;
   if(whichModel == -1)
        return;
   if(!ModelInfo[whichModel].created)
        return;
   int c = GPOINTER_TO_INT(data);
   AllCentr[whichModel].at(c).visible = (gtk_toggle_button_get_active(but) != 0);
   gl->Redraw();
}

void toggle_centlvis(GtkToggleButton *but, gpointer data)
{
using namespace Centroids;
   if(!GLWINDOW)
      return;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
   int whichModel = GetActiveModel() - 1;
   if(whichModel == -1)
        return;
   if(!ModelInfo[whichModel].created)
        return;
   int c = GPOINTER_TO_INT(data);
   AllCentr[whichModel].at(c).showlab = (gtk_toggle_button_get_active(but) != 0);
   gl->Redraw();
}

void on_change_cent_transp(GtkRange *but, gpointer data)
{
using namespace Centroids;
   if(!GLWINDOW)
      return;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
   int whichModel = GetActiveModel() - 1;
   if(whichModel == -1)
        return;
   if(!ModelInfo[whichModel].created)
        return;
   int c = GPOINTER_TO_INT(data);
   AllCentr[whichModel].at(c).transparency = (int) gtk_range_get_value(but);
   gl->Redraw();
}

static void lab_changed(GtkEntry *entry, gpointer data)
{
using namespace Centroids;
   if(!GLWINDOW)
      return;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
   int whichModel = GetActiveModel() - 1;
   if(whichModel == -1)
        return;
   if(!ModelInfo[whichModel].created)
        return;
   int c = GPOINTER_TO_INT(data);
   CopiaStringan(AllCentr[whichModel].at(c).label, lenLab,
         gtk_entry_get_text(entry), lenLab-1);
   AllCentr[whichModel].at(c).label[lenLab-1] = 0;
   gl->Redraw();
}

#ifdef IN_GTK3
void ChangedCentColor(GtkWidget *cb, gpointer data)
#else
void ChangedCentColor(GtkWidget *cb, GtkStyle *style, gpointer data)
#endif
{
using namespace Centroids;
   if(!GLWINDOW)
      return;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");

   int whichModel = GetActiveModel() - 1;
   if(whichModel == -1)
        return;
   if(!ModelInfo[whichModel].created)
        return;

   float color[4];
   mycolor_button_get_color(cb, color);
   int c = GPOINTER_TO_INT(data);
   for(int i=0; i<4; i++)
       AllCentr[whichModel].at(c).colore[i] = color[i];
   gl->Redraw();
}

gboolean selected_centroid(GtkWidget *wid, GdkEvent *ev, gpointer data)
{
using namespace Centroids;
   if(!GLWINDOW)
       return FALSE;
   GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
   unsigned int quale = GPOINTER_TO_INT(data);
   int whichModel = GetActiveModel() - 1;
   if(whichModel == -1)
         return FALSE;
   if(!ModelInfo[whichModel].created)
         return FALSE;
   if(AllCentr[whichModel].empty())
         return FALSE;
   if(AllCentr[whichModel].at(quale).selected)
   {
       AllCentr[whichModel].at(quale).selected = false;
       widget_set_unselected(wid);
       gl->Redraw();
       return TRUE;
   }
   GtkWidget *dlg = gtk_widget_get_toplevel(wid);
   for(size_t i=0; i<AllCentr[whichModel].size(); i++)
   {
        std::stringstream base;
        base << i;
        std::string nome;
        nome.assign("eventbox");
        nome.append(base.str());
        AllCentr[whichModel].at(i).selected = (i == quale);
        GtkWidget *w = get_widget(dlg, nome.c_str());
        widget_set_unselected(w);
   }
   widget_set_selected(wid);
   gl->Redraw();
   return TRUE;
}


void FillCentroidsDlg()
{
using namespace Centroids;
  const char *titles[] = {"Number", "Show", "Label", "Show label",  
                           "Colour", "Transparency"};
  int ncolumns = (int) (sizeof(titles)/sizeof(titles[0]));
  GtkWidget *father = get_widget(GLWINDOW, "win_centroids");
  GtkWidget *list = get_widget(father, "list_centroids");
  GtkSizeGroup **szgroups = g_new0(GtkSizeGroup *, ncolumns);
  remove_children(list);
  int whichModel = GetActiveModel() - 1;
  if(whichModel == -1)
     return;
  if(AllCentr[whichModel].empty())
     return;
  int nCentroids = (int) AllCentr[whichModel].size();
  std::vector<jav_centroid_type> Centr = AllCentr[whichModel];
  int hspace = 6;
  
  GtkWidget *align1 = vbox_new(FALSE, 0); 
  gtk_container_add(GTK_CONTAINER(list), align1);
  GtkWidget *vbox2 = hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(align1), vbox2, FALSE, FALSE, 0);
  for(int i=0; i<ncolumns; i++)
  {
      szgroups[i] = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
      GtkWidget *label = LABEL_NEW(titles[i], MyBLUE);
      set_alignment(label, 0.5, 0.0);
      gtk_box_pack_start(GTK_BOX(vbox2), label, TRUE, TRUE, hspace);
      gtk_size_group_add_widget(szgroups[i], label);
  }
  for(int i=0; i<nCentroids; i++)
  {
      std::stringstream base;
      base << i;
      std::string nome;
      GtkWidget *ebox = gtk_event_box_new ();
//      if(i == 0)
//         widget_set_selected (ebox);
      gtk_box_pack_start (GTK_BOX (align1), ebox, FALSE, FALSE, 4);
      gtk_event_box_set_visible_window(GTK_EVENT_BOX(ebox), TRUE);
      gtk_widget_add_events(ebox, GDK_BUTTON_PRESS_MASK);
      gtk_widget_set_name(ebox, "myebox");
      nome.assign("eventbox");
      nome.append(base.str());
      g_object_set_data(G_OBJECT(father), nome.c_str(), ebox);
      g_signal_connect (G_OBJECT (ebox), "button_press_event",
                      G_CALLBACK (selected_centroid), GINT_TO_POINTER(i));
      vbox2 = hbox_new(FALSE, 0);
      gtk_container_add (GTK_CONTAINER (ebox), vbox2);
      int k = 0;
//Number
      char *lab1 = g_strdup_printf("%2d", i+1);
      GtkWidget *label = LABEL_NEW(lab1, MyBLUE);
      set_alignment(label, 0.5, 0.5);
      gtk_box_pack_start(GTK_BOX(vbox2), label, TRUE, TRUE, hspace);
      gtk_size_group_add_widget(szgroups[k], label);
      k++;

//Visible
      nome.assign("visible");
      nome.append(base.str());
      label = gtk_check_button_new();
      g_object_set_data(G_OBJECT(father), nome.c_str(), label);
#ifdef IN_GTK3
      gtk_widget_set_hexpand(label, FALSE);
      gtk_widget_set_vexpand(label, FALSE);
      g_object_set(G_OBJECT(label), "halign", GTK_ALIGN_CENTER, NULL);
      g_object_set(G_OBJECT(label), "valign", GTK_ALIGN_CENTER, NULL);
      gtk_box_pack_start(GTK_BOX(vbox2), label, TRUE, TRUE, hspace);
      gtk_size_group_add_widget(szgroups[k], label);
#else
      GtkWidget *align = gtk_alignment_new(0.5, 0.5, 0., 0.);
      gtk_box_pack_start(GTK_BOX(vbox2), align, TRUE, TRUE, hspace);
      gtk_container_add(GTK_CONTAINER(align), label);
      gtk_size_group_add_widget(szgroups[k], align);
#endif
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(label), (Centr.at(i).visible == true));
      g_signal_connect(G_OBJECT(label), "toggled", G_CALLBACK(toggle_centvis),
          GINT_TO_POINTER(i));
      k++;

//Label
      std::string lab = g_strdup(Centr.at(i).label);
      label = gtk_entry_new();
      set_width_chars(GTK_ENTRY(label), lenLab);
      gtk_entry_set_text(GTK_ENTRY(label), lab.c_str());
      gtk_box_pack_start(GTK_BOX(vbox2), label, TRUE, TRUE, hspace);
      g_signal_connect(G_OBJECT(label), "activate", G_CALLBACK(lab_changed),
          GINT_TO_POINTER(i));
      gtk_size_group_add_widget(szgroups[k], label);
      k++;

//Show label
      nome.assign("showlab");
      nome.append(base.str());
      label = gtk_check_button_new();
      g_object_set_data(G_OBJECT(father), nome.c_str(), label);
#ifdef IN_GTK3
      GtkWidget *align = gtk_frame_new(NULL);
      gtk_frame_set_shadow_type(GTK_FRAME(align), GTK_SHADOW_NONE);
      gtk_box_pack_start(GTK_BOX(vbox2), align, TRUE, TRUE, hspace);
      gtk_widget_set_hexpand(label, FALSE);
      gtk_widget_set_vexpand(label, FALSE);
      g_object_set(G_OBJECT(label), "halign", GTK_ALIGN_CENTER, NULL);
      g_object_set(G_OBJECT(label), "valign", GTK_ALIGN_CENTER, NULL);
      gtk_size_group_add_widget(szgroups[k], align);
      gtk_container_add(GTK_CONTAINER(align), label);
#else
      align = gtk_alignment_new(0.5, 0.5, 0., 0.);
      gtk_box_pack_start(GTK_BOX(vbox2), align, TRUE, TRUE, hspace);
      gtk_container_add(GTK_CONTAINER(align), label);
      gtk_size_group_add_widget(szgroups[k], align);
#endif
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(label), (Centr.at(i).showlab == true));
      g_signal_connect(G_OBJECT(label), "toggled", G_CALLBACK(toggle_centlvis),
          GINT_TO_POINTER(i));
      k++;
//Colore
      nome.assign("colore");
      nome.append(base.str());
      label = mycolor_button(Centr.at(i).colore, "Pick a color to be used for polyhedra");
      g_object_set_data(G_OBJECT(father), nome.c_str(), label);
#ifdef IN_GTK3
      gtk_widget_set_hexpand(label, FALSE);
      gtk_widget_set_vexpand(label, FALSE);
      g_object_set(G_OBJECT(label), "halign", GTK_ALIGN_CENTER, NULL);
      g_object_set(G_OBJECT(label), "valign", GTK_ALIGN_CENTER, NULL);
      gtk_box_pack_start(GTK_BOX(vbox2), label, FALSE, FALSE, hspace);
      gtk_size_group_add_widget(szgroups[k], label);
#else
      align = gtk_alignment_new(0.5, 0.5, 0., 0.);
      gtk_box_pack_start(GTK_BOX(vbox2), align, FALSE, FALSE, hspace);
      gtk_container_add(GTK_CONTAINER(align), label);
      gtk_size_group_add_widget(szgroups[k], align);
#endif
      g_signal_connect(G_OBJECT(label), COLOR_CHANGED, G_CALLBACK(ChangedCentColor), GINT_TO_POINTER(i));
      k++;
//Trasparenza
      nome.assign("Transparency");
      nome.append(base.str());
      label = hscale_new_with_range(0, 100, 1);
      gtk_scale_set_digits(GTK_SCALE(label), 0);
      gtk_scale_set_value_pos(GTK_SCALE(label), GTK_POS_BOTTOM);
      gtk_scale_set_draw_value(GTK_SCALE(label), TRUE);
      gtk_range_set_value(GTK_RANGE(label), Centr.at(i).transparency);
      g_object_set_data(G_OBJECT(father), nome.c_str(), label);
      gtk_box_pack_start(GTK_BOX(vbox2), label, TRUE, TRUE, hspace);
      int hnd = g_signal_connect(G_OBJECT(label), "value-changed", G_CALLBACK(on_change_cent_transp), GINT_TO_POINTER(i));
      g_object_set_data(G_OBJECT(label), "hnd", GINT_TO_POINTER(hnd));
      gtk_size_group_add_widget(szgroups[k], label);
  }
  gtk_widget_show_all(list);
}

void on_centroids(gpointer item, gpointer data)
{
   GtkWidget *window = (GtkWidget *) data;
   int whichModel = GetActiveModel() - 1;
   if(whichModel == -1)
      return;
   if(!ModelInfo[whichModel].created)
        return;
   GtkWidget *dialog1 = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dialog1), "Centroids");
   gtk_container_set_border_width(GTK_CONTAINER(dialog1), 8);
   g_object_set_data(G_OBJECT(window), "win_centroids", dialog1);
#ifndef IN_GTK3
   gtk_widget_set_size_request(dialog1, 780, -1);
#endif
   GtkWidget *vbox1 = gtk_dialog_get_content_area(GTK_DIALOG(dialog1));
   gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog1), TRUE);
   gtk_window_set_transient_for(GTK_WINDOW(dialog1), GTK_WINDOW(window));
   GtkWidget *scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
   gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (scrolledwindow1), 0);
   gtk_widget_set_size_request(scrolledwindow1, -1, 300);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1),
        GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
   GtkWidget *viewport = gtk_viewport_new(NULL, NULL);
   gtk_viewport_set_shadow_type (GTK_VIEWPORT(viewport), GTK_SHADOW_NONE);
   gtk_container_add (GTK_CONTAINER (scrolledwindow1), viewport);
#if 0
   GdkColor *bgcolor = getBgColor(dialog1);
   widget_modify_bg(viewport, GTK_STATE_NORMAL, bgcolor);
#endif
   GtkWidget *vbox = vbox_new(FALSE, 0);
   gtk_container_add (GTK_CONTAINER (viewport), vbox);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
   g_object_set_data(G_OBJECT(dialog1), "list_centroids", vbox);

   GtkWidget *button = myimage_button("_Find Centroids", "find3.png");
   gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), button, 1);
   (void) g_signal_connect(G_OBJECT (button), "clicked",
          G_CALLBACK(find_centroids), data);
   gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_widget_get_parent(button)),
      GTK_BUTTONBOX_SPREAD);
   button = myimage_button("    _Close", "cancel.png");
   gtk_dialog_add_action_widget (GTK_DIALOG (dialog1), button, 0);
   (void) g_signal_connect(G_OBJECT (button), "clicked",
          G_CALLBACK(close_centroids), data);

   (void) g_signal_connect_swapped(G_OBJECT(dialog1), "delete_event",
         G_CALLBACK(gtk_button_clicked), G_OBJECT(button));
    (void) g_signal_connect (G_OBJECT (dialog1), "delete_event",
                       G_CALLBACK(true_cb), NULL);
   gtk_widget_show_all(dialog1);
   FillCentroidsDlg();
}

void DrawCentroids(GLWindow *gl)
{
using namespace Centroids;
    PangoFontDescription *fdesc = pango_font_description_from_string(gl->fontname);
    for (int m=0; m<gl->nModels; m++)
    {
         if(AllCentr[m].empty())
            continue;
         int nCentroids = (int) AllCentr[m].size();
         for(int i=0; i<nCentroids; i++)
         {
             if(!AllCentr[m].at(i).visible)
                continue;
             AllCentr[m].at(i).Draw();
             if(AllCentr[m].at(i).selected)
             {
                 gl->SetLightForSelected(1);
                 float add = (float)(0.3 / gl->deltav);
                 AllCentr[m].at(i).DrawSelected(add);
                 gl->ResetLightForSelected();
             }
             if(AllCentr[m].at(i).showlab)
             {
                PangoRectangle *rect1 = get_text_rect(fdesc, 
                     AllCentr[m].at(i).label);
                double d = radN * (gl->sc_atoms*gl->Scale*sc_factor)/SCALA;
                double delta[3] = {
                    d * gl->xoffset * npixels / 2,
                    d * gl->yoffset * npixels / 2 - rect1->height/2,
                    d};
                AllCentr[m].at(i).DrawLabel(delta, fdesc,
                       ModelInfo[m].labelcolor);
             }
          }
     }
     pango_font_description_free(fdesc);
}

void NormalizzaCentroids(GLWindow *gl)
{
using namespace Centroids;
    radN = radC/gl->deltav;
    for (int m=0; m<gl->nModels; m++)
    {
         if(AllCentr[m].empty())
            continue;
         int nCentroids = (int) AllCentr[m].size();
         for(int i=0; i<nCentroids; i++)
         {
             vet3 xyzo = gl->FractToCart(AllCentr[m].at(i).xyz);

             vet3 a = gl->Trasforma(xyzo);
             for(int j=0; j<3; j++)
                 AllCentr[m].at(i).xyzN[j] = (float) a.v[j];
         }
    }
}

}   // end namespace GLextmr
extern "C" void start_centroids_()
{
using namespace Centroids;
using namespace GLextmr;
     int whichModel = GetActiveModel() - 1;
     if(whichModel == -1)
        return;
     AllCentr[whichModel].clear();
}

extern "C" void passa_centroid_(int *ind, float xyz[3], int *natoms, int *at)
{
using namespace Centroids;
using namespace GLextmr;
     jav_centroid_type c;
     //float col[] = {1, 0.8f, 0.2f, 1};
     float col[] = {0.8f, 1, 0.8f, 1};
     int whichModel = GetActiveModel() - 1;
     if(whichModel == -1)
        return;

     for (int i=0; i<3; i++)
         c.xyz[i] = xyz[i];
     c.nat = *natoms;
     c.atoms = g_new0(int, c.nat);
     for (int i=0; i<c.nat; i++)
          c.atoms[i] = at[i] - 1;
     c.visible = true;
     c.showlab = true;
     c.transparency = 50;
     c.selected = 0;//(*ind == 1);
     memcpy(c.colore, col, 4*sizeof(float));
     std::string strLab, strTemp;
     strTemp.assign(ModelInfo[whichModel].atm[c.atoms[0]].lab);
     strLab = trim(strTemp);
     for (int i=1; i<c.nat; i++)
     {
         strLab.append(",");
         strTemp.assign(ModelInfo[whichModel].atm[c.atoms[i]].lab);
         strLab.append(trim(strTemp));
     }
     CopiaStringa(c.label, lenLab, strLab.substr(0, lenLab-1).c_str());
//g_strdup_printf("%d atoms", c.nat));
     AllCentr[whichModel].push_back(c);
}

extern "C" void end_centroids_()
{
using namespace Centroids;
using namespace GLextmr;
     int whichModel = GetActiveModel() - 1;
     if(whichModel == -1)
        return;
     if(!ModelInfo[whichModel].created)
        return;
     FillCentroidsDlg();
     GLWindow *gl = (GLWindow *)g_object_get_data(G_OBJECT(GLWINDOW), "glwindow");
     gl->RiNormalizza();
     gl->Redraw();
}
