#include <cstring>
#include "libcomune.h"
#include "cry.h"

void Crystal::FillCrystal(int nats, int nels, crystal_t cr, atom_type *atm, element_type *el)
{
    memcpy(cell, cr.cell, 6*sizeof(float));
    memcpy(cellcod, cr.cellcod, 6*sizeof(int));
    cname = new string(cr.cname, 300);
    spgname = new string(cr.spgname, 16);
    scal = cr.scal;
    scalcod = cr.scalcod;
    kopcorr = cr.kopcorr;
    memcpy(hklpo, cr.hklpo, 3*sizeof(int));
    gcode = cr.gcode;
    gpar = cr.gpar;
    riding_model = cr.riding_model;
    adp_factor = cr.adp_factor;
    elem = el;
    at = atm;
    natoms = nats;
    nelems = nels;
    nconstr = 0;
    vconstr = NULL;
    dispo = NULL;
    selected_r = NULL;
    ind_Con = NULL;
}

void Crystal::PrintCrystal()
{
      g_print("Name: %s\n", trim(*cname).c_str());
      g_print("Cell: ");
      for(int i=0; i<6;i++)
          g_print("%12.5f ",cell[i]);
      g_print("\n");
      g_print("Code: ");
      for(int i=0; i<6;i++)
          g_print("%2i ",cellcod[i]);
      g_print("\n");
      g_print("Space group: %s\n", spgname->c_str());
      g_print("Scale: %0.3f Code: %d\n",scal,scalcod);
      for(int i=0; i<natoms; i++)
         g_print("%d) %.4s %10.4f %10.4f %10.4f %10.3f %8.3f\n", i+1,
           at[i].lab,
           at[i].xc[0], at[i].xc[1],
           at[i].xc[2], at[i].biso,
           at[i].och);
}

string Crystal::GetElemName(int n)
{
     string nome(elem[n].name, 20);
     return (trim(nome));
}

string Crystal::GetAtomName1(int n)
{
     string nome(at[n].lab, 16);
     return (trim(nome));
}

void Crystal::add_constraint(int code, int npar, int ref, int vpar[],
             int vat[], float coef[], float sumc, int i, int typec, int npari)
{
   int j;

   vconstr[i].code = code;
   vconstr[i].npar = npar;
   vconstr[i].ref  = ref;
   vconstr[i].typec  = typec;
   vconstr[i].npari  = npari;
   vconstr[i].sumc = sumc;
   vconstr[i].vpar = g_new0(int, vconstr[i].npar);
   vconstr[i].vat  = g_new0(int, vconstr[i].npar);
   vconstr[i].coef = g_new0(float, vconstr[i].npar);
   for(j=0;j<vconstr[i].npar; j++)
   {
       vconstr[i].vpar[j] = vpar[j];
       vconstr[i].vat[j] = vat[j];
       vconstr[i].coef[j] = coef[j];
       if(vconstr[i].code == 1 && vconstr[i].typec == 1)
          dispo[vconstr[i].vat[j]-1] = 0;  // only for atoms in user adp constraint 
   }
}

void Crystal::del_constraint(int i)
{
    int j;
    for(j=i+1; j<nconstr; j++)
        memcpy(&(vconstr[j-1]), &(vconstr[j]), sizeof(constraint_type));
}

int  Crystal::par_in_constraint(int kat, int npar)
{
//  Controlla se un atomo e' implicato in un constraint
   int i, j;

   for(i=0; i<nconstr; i++)
   {
       for(j=0; j<vconstr[i].npar; j++)
       {
         if ((vconstr[i].vpar[j] == npar) &&  (vconstr[i].vat[j] == kat))
            return (i+1);
       }
   }
   return 0;
}

void Crystal::SetNconstr(int nc)
{
   nconstr = nc;

   if(nconstr)
      vconstr = g_new0(constraint_type, nconstr);
   ind_Con = g_new0(int, natoms*3);
   dispo = g_new0(int, natoms);
   selected_r = g_new0(int, natoms);
   for(int i=0; i<natoms; i++)
   {
       dispo[i] = 1;
       selected_r[i] = 0;
   }

}

void Crystal::free_constraints()
{
   int i;
   if(vconstr)
   {
      for(i=0; i<nconstr; i++)
      {
          g_free(vconstr[i].vpar);
          g_free(vconstr[i].vat);
          g_free(vconstr[i].coef);
      }
      g_free(vconstr);
      vconstr = NULL;
   }
   if(dispo)
   {
      g_free(dispo);
      dispo = NULL;
      g_free(selected_r);
      selected_r = NULL;
   }
   g_free(ind_Con);
}

bool Crystal::Reallocconstr()
{
   vconstr = (constraint_type *) g_try_realloc(vconstr,
             (nconstr+1)*sizeof(constraint_type));
   return(vconstr != NULL);
}
