MODULE datamod

   implicit none

CONTAINS

   subroutine gener(wave,iflagref,kpr)
   USE Riflessi
   USE General, only:ihmx,rhomin,rhomax,dtor,lo,cell,nrefl,knw,spacegr,alambda
   USE counts, only: dvalue
   USE variables, only: cryst,ref,dataset
   USE PatternRef, only: thmax,thmin
   USE reflection_type_util
   USE testmodel, only: atmpub
   implicit none
   real, dimension(:), intent(in)                   :: wave
   integer, intent(out)                             :: iflagref
   logical, intent(in)                              :: kpr
   integer                                          :: i,ier
   type(reflection_type), dimension(:), allocatable :: refob
   integer                                          :: nrefob
!
   rhomax = 1. / (2*dvalue(thmax,alambda))**2
   rhomin = 1. / (2*dvalue(thmin,alambda))**2
!
   iflagref = 0
   call create_reflections(thmin,thmax,cell(:6),spacegr,wave,refob,nrefob,ihmx)
!
   call AllocaRiflessi(2)
   nrefl = nrefob
   if (nrefl == 0) return
   call AllocaRiflessi(1)
   if (knw > 0) then
       call manage_filefc(refob,ier)
       if (ier /= 0) knw = 0
   endif
   do i=1,nrefl
      IndHKL(i,1:3) = refob(i)%hkl(:)
      IndHKL(i,4) = 1
!corr      IndHKL(i,5) = refob(i)%phv
      IndHKL(i,5) = refob(i)%jcode + refob(i)%phv*32
      SetFob(i,1) = 0.0
      SetFob(i,2) = 0.0
      SetFob(i,3) = 0.0
      SetFob(i,4) = 0.0001
      TThet (i)   = refob(i)%tthd(1)
      Mult  (i)   = refob(i)%m
      WHM   (i)   = 0.0
   enddo
   if (knw > 0) fxsav(:) = refob(:nrefl)%fv   ! :nrefl because refob is not reallocated
   nrefl = nrefob  ! set global varible nrefl after AllocaRiflessi
!
   if (nrefl > 3000) iflagref = 1             ! segnala che ci sono + di 3000 riflessi
   if (kpr) then
       write(lo, '(16x,"*",7x,"Generated reflections information",10x,"*")')
       write(lo, '(16x,"*",50x,"*")')
       write(lo, '(16x,"*",9x,"rho min         = ",f8.5,15x,"*")') rhomin
       write(lo, '(16x,"*",9x,"rho max         = ",f8.5,15x,"*")') rhomax
       write(lo, '(16x,"*",9x,"d (resolution)  = ",f8.5,15x,"*")') dvalue(thmax,alambda)
       write(lo,'(16x,"*",9x,"reflections nr. = ",i8,15x, "*",/,16x,"*",9x,"h k l (max)     = ",3i4,11x,"*",/,16x,52("*"))') &
       nrefl,(ihmx(i),i=1,3)
   endif
   call cryst(1)%set_reflections(refob)
   if (knw > 0) call cryst(1)%set_as_known(atmpub,dataset(1)%radtype)
   call copy_ref(ref,refob)
   end subroutine gener

!-----------------------------------------------------------------------

   subroutine manage_filefc(refob,ier)   
   USE General, only:nrefl, alambda, fname, FC_FILE, dtor, neutro, celem, lo, spacegr
   USE filereading
   USE fileutil
   USE TestModel
   USE reflection_type_util
   USE atom_type_util
   USE strutil
   USE elements, only:H_at
   USE connect_mod
   USE errormod
   USE fra_frm
   USE spginfom
   USE variables, only: cryst
   USE gen_frm
   USE unit_cell
   implicit none
   type(reflection_type), dimension(:), intent(inout) :: refob
   integer, intent(out)                               :: ier
   type(reflection_type), allocatable, dimension(:)   :: reffc
   type(error_type)                                   :: error
   integer                                            :: nrfc
   integer                                            :: fase
   integer                                            :: i,j
   type(bond_type), dimension(:), allocatable         :: legpub
   integer, dimension(3)                              :: hkl
   integer                                            :: is
   type(spaceg_type)                                  :: spgfc
   type(cell_type)                                    :: cellpub
   logical                                            :: is_bfac,is_occ,has_symmetry
!
   ier = 0
   if (s_eqi(get_extension(fname(FC_FILE)),'fc')) then    ! read F from .fc file
       refob(:)%fv = 0
       call read_reflection_file(fname(FC_FILE),reffc,nrfc,'fc',error)
       if (error%signal) then
           call error%print()
           ier = 1
       else
           do i=1,nrfc
              loop_sym: do is=1,spacegr%nsymop
                 !hkl(:) = matmul(reffc(i)%hkl,kmat(is,:,:))
                 hkl(:) = matmul(reffc(i)%hkl,spacegr%symop(is)%rot)
                 do j=1,nrefl
                    if (all(hkl == refob(j)%hkl)) then
                        fase=mod(reffc(i)%ph,360)
                        if(fase <= 0)fase=fase+360
                        refob(j)%fv = reffc(i)%fc
                        refob(j)%phv=fase
!corr                        refob(j)%jcode=refob(j)%jcode+fase*32
                        exit loop_sym
                    endif
                 enddo
              enddo loop_sym
           enddo
!
!          read file .fra
           call read_fra_file(file_rem_ext(fname(FC_FILE))//'.fra',atmpub,is_bfac,is_occ,error)
           if (error%signal) then
               write(lo,'(a)')trim(error%message)
               write(lo,'(a)')'Match between Expo model and fragment is not possible!'
               return
           endif
           call specie_from_ptab(atmpub,celem)
       endif
   else                                                   ! try to read structure file
       allocate(reffc(nrefl))
       reffc(:) = refob(:nrefl)
       reffc(:)%slaq = (sin(0.5*refob(:nrefl)%tthd(1)*dtor)/alambda)**2
       !has_symmetry = is_cell_defined_list()
       !  write(0,*)'CELL P=',cell(1:6)
       has_symmetry = .false.
         !write(0,*)'CELL P=',cryst(1)%get_cellpar()
         !write(0,*)'CELL P=',cryst(1)%get_cellpar()
       !call import_structure(fname(FC_FILE),0,atmpub,legpub,celem,cellfc,spgfc,.false.,has_symmetry,error)
       call import_crystal(fname(FC_FILE),ALL_FILES,atmpub,legpub,celem,cellpub,spgfc,has_symmetry,neutro,.false.,error)
       call cryst(1)%set_scattering(celem) ! update elements, published model could have different contents
         !write(0,*)'CELL D=',cell(1:6)
         !write(0,*)'CELL D=',cryst(1)%get_cellpar()
       if (error%signal) then
           ier = 1
       else           ! now compute fc
           call fcalcang(reffc,atmpub,spacegr,celem)
           do i=1,nrefl
              fase=mod(reffc(i)%ph,360)
              if(fase <= 0)fase=fase+360
              refob(i)%fv=reffc(i)%fc
              refob(i)%phv=fase
!corr              refob(i)%jcode=refob(i)%jcode+fase*32
           enddo
           if (neutro == 0) call remove_atoms_from_list(atmpub,atmpub%ptab,H_at)  ! remove H atoms
       endif
   endif
!
   end subroutine manage_filefc

END MODULE datamod

   subroutine data1(active_file,ier)
   USE Conteggi
   USE VIEW
   USE ssmoothing
   USE General
   USE Symmetry
   USE PatternRef
   USE filereading
   USE messagemod
   USE Molcom, only:jscreen 
   USE fileutil
   USE peak_mod
   USE datasetmod
   USE datautil
   USE variables
   USE indexing
   USE datamod
   USE mpi_prog
   USE plotstyle, only:styledata
   !use datautil
   !use errormod
   implicit none
   common /ksearch/ ksearch
   integer :: ksearch
   common /spacegroup/iesspace,shiftzero
   integer :: iesspace
   real :: shiftzero
   common /centering/CellType,CellAxis,nruntre,iesdircell
   character CellType,CellAxis*3
   integer :: nruntre,iesdircell
   common /step_data/istog
   integer :: istog
   integer :: ier, iflagref
   integer :: iesdirsear
   integer             :: ismoo,nsmoo,npsmoo
   logical             :: modify_step
   integer, intent(in) :: active_file
   common/exterfile/fname3ex
   character(len=80) :: fname3ex
   common/iesext/iesext
   integer :: iesext
   common/itypepeak/itypep
   integer :: itypep,ndat
   type(dataset_type) :: datas
   logical :: okrad
   integer :: nwave,exitcode
   real, dimension(3) :: wave,ratio
   logical :: scaledata
   real    :: scaleval
   !type(error_type) :: err
!
   call write_inte_routine('data')
!
   nsym=0
   nrefl=0
   rhomax=0.0
   lpseuv(:)=0
   iflag=0
!
   call dirdat(filehpow,istog,ksearch,ismoo,nsmoo,npsmoo,active_file,  &
                modify_step,nwave,wave,ratio,iesdirsear,okrad,scaledata,scaleval,ier)
   if (ier < 0) go to 90
!
   if (active_file == REFL_FILE) then
       if (nwave > 0) alambda = wave(1)
       return
   endif
!
!  Leggi file di dati in countp
   call datas%set_wave(nwave,wave,ratio)
   call load_datafile(datas,filehpow,jscreen>0,ier,filetypepow)
   if (ier /= 0) go to 90
   datas%style = styledata%get(ndataset(dataset)+1)
   call push_back_dataset(dataset,datas)
   ndat = ndataset(dataset)
   if (modify_range) call dataset(ndat)%resize(thmin,thmax)
   if (okrad) dataset(ndat)%radtype = neutro
   if (scaledata) call dataset(ndat)%scale_data(scaleval)
   if (dataset(ndat)%nwave > 0) then
       alambda = dataset(ndat)%wave(1)
       call init_indexation(dataset(ndat)%wave(1))
   else
       call MsgWinErr('ERROR','No wavelength supplied',ERR_WINDOW,exitcode)
       ier = -2
       go to 90
   endif
!
!  scarica in theta_int
   call dataset_to_thetaint(dataset(ndat))
!
!  Check peak search conditions provided by user by directive search
   if (iesdirsear == 1) call check_pkcond(pkcond,thmin,thmax) 
!
   call write_message_pattern(filehpow)
!
   if(ismoo.gt.0)theta_int(:,2)=savgol_smooth(theta_int(:,2),nsmoo,nsmoo,0,npsmoo)
!
   if (mpi_prog_rank == mpi_prog_master) then
       call dataset(ndat)%prn(lo,iesdircell==0)
   endif
!
   if (iesdircell == 1) then  ! cell defined
       if(.not.cryst(1)%is_spg()) then
!
!         cell defined and procedure find space required
          call gener_findspace(ier) ! generate in laue class
          if (ier.lt.0) go to 90
       endif
       call gener(dataset(ndat)%wave,iflagref,mpi_prog_rank==mpi_prog_master)
       call write_message('Reflections nr.: ',inum=nrefl,pos=6)
   endif
   call ysfr37(npunti)
   if(jscreen > 0) then
      call vedinew(8,1,npunti)
      call write_message('Press " Pattern " to work on the pattern or '   &
                         //'"Next" to go on',pos=7)
   endif                       
   if (iesext > 0) then
!
!      Load peak from external file specified by directive 'extefile'
       if (itypep == 1) then
           call LoadPeaks(fname3ex,ier,2)   ! d values (default)
       else
           call LoadPeaks(fname3ex,ier,1)   ! 2theta values
       endif
       if (ier == 0) call peak_set(pkind,code=0) ! this disable the refinement of peaks
   endif

   !call combine_dataset(dataset,"tmp.xy",err)

   return
90 continue

   return
   end

!---------------------------------------------------------------------

   subroutine dirdat(fileh,istog,ksearch,ismoo,nsmoo,npsmoo,active_file,  &
                     modify_step,nwave,wave,ratio,iesdirsear,okrad,scaledata,scaleval,ier)
   USE General
   USE Symmetry
   USE filereading
   USE commandsmod
   USE PatternRef
   USE indexing
   USE elements
   USE proginterface
   USE strutil, only: string_locate, getnum, err_string
   USE spginfom
   USE peak_mod, only: pkcond
   USE comandiexpo
   USE unit_cell
   USE sinterface
   USE variables, only: cryst, dataset
   USE crystal_phase
   USE datasetmod, only: ndataset
   USE mpi_prog

   implicit none
   character(len=*)                :: fileh
   integer, intent(out)            :: ksearch,iesdirsear,ismoo,nsmoo,npsmoo
   integer, intent(out)            :: istog
   logical, intent(out)            :: modify_step,okrad,scaledata
   integer, intent(out)            :: nwave
   real, intent(out)               :: scaleval
   real, dimension(3), intent(out) :: wave,ratio
   common /cpseud_new/ ivet_new(3,4),percme   !!!!,esqv(3000)
   integer :: ivet_new
   real :: percme   !,esqv
   character dire*80,diret*80     !!!!!,formt*80
   character(len=len(card)) :: line
   common/errer/erreprof,erreprofw,erre_old,erref,errerif
   real :: erreprof,erreprofw,erre_old,erref,errerif
   common /ureq1/ ncycl,nroset,nundset,nsec,jback,uvwt(8),itry,pfwhm,kequirip
   integer :: ncycl,nroset,nundset,nsec,jback,itry,kequirip
   real :: uvwt,pfwhm
   integer :: ivet(40),kshif(3),kistp(8)
   real :: vistp(8), vet(40)
   character(len=4), dimension(8) :: isotope
   common /peak_noghost/noghost
   integer :: noghost
   common/iesext/iesext
   integer :: iesext
   common/corr2th/thstepcor
   real :: thstepcor
   common /centering/CellType,CellAxis,nruntre,iesdircell
   integer :: nruntre,iesdircell
   character CellType,CellAxis*3
   common /direx/ncycle,icycle,nosing,izero ,zeroprof,asym,scal,frac   &
                 ,noint,ialt,sigma,if92,nocell
   integer :: ncycle,icycle,nosing,izero,noint,ialt,if92,nocell
   real :: zeroprof,asym,scal,frac,sigma
   common/itypepeak/itypep
   integer :: itypep
   common /profp/dumpp,nprof,ipol
   real :: dumpp
   integer :: nprof,ipol
   common /definespac/iesspacesav,itrial,irunextra,nlin
   integer :: iesspacesav,itrial,irunextra,nlin
   common /spacegroup/iesspace,shiftzero
   integer :: iesspace
   real :: shiftzero
   common /misurato/ misu
   integer :: misu
   common/spaceforplot/numsymb,spacesymb(400),figsymb(400),isymb(400),numerosymb(400)
   integer :: numsymb,isymb,numerosymb
   real :: figsymb
   common/spaceindexing /ispaceindexing,fspaceindex(660),ntotmult1,ntotmult2,      &
                         percind(660),errespac(660),lauecell(660)
   integer :: ispaceindexing,ntotmult1,ntotmult2
   real :: fspaceindex,percind,errespac
   character*20 lauecell
   real, dimension(13) :: celltemp
   character*20 spacesymb     
   common/exterfile/fname3ex
   character(len=80) :: fname3ex
   logical :: is_indexing
   integer, intent(in) :: active_file 
   integer :: i,j,iv,ier,lend,kistog,ic,icmax
   integer :: Lung, nExitCode,kpos,ksh,lix1,lix2,lix3,lix7
   integer :: iesnorefine,iesdircont,nistp,ishif,lenp
   integer :: ier_spg
   logical :: symm_required
   integer :: nrt
!
! nprop=1 pearson
! nprop=2 gussian
! nprop=3 lorentzian
! nprop=4 pseudo-voigt
!
!pgMAD280907
   kmad = 0
!corr   misu = 1          ! se misu = 0 non utilizza il wilson plot
!corr                     ! se misu = 1 utilizza il wilson plot
!corr   kmad = 0          ! se kmad <> 0  allora i dati sono per Multiple Anomalous Dispersion
!corr   nWave = 0         ! numero lunghezze d'onda per Multiple Anomalous Dispersion
!corr   do i=1,5
!corr      normm(i)=0      ! se norm(i)=1 si normalizza la i-esima l.d.o.
!corr   enddo
!corr   ifasTot = 0        !pgMAD050208
!corr   ispaceindexing = 0
!corr   kmad = 0
!corr   ifEsuSigma = 0     !pgMAD130109
!corr   eMadMax = 9999.
!corr   NumFiltri = 0
!corr   do i=1,10
!corr      nFiltro(i) = -1   ! esclusi per default per default
!corr   end do
!corr   nFiltro(2) = 0       ! filtro sulla risoluzione automatico per default
!corr   numbWL = 0       
!pgMAD280907      
!
!  Output variables
   modify_step = .false.
   ismoo=0
   nsmoo=5
   npsmoo=3
!
!  Local variables
   iesnorefine = 0
   celltemp(:) = 0.0
   iesdircont = 0
   kistog=0
   nistp=0
   ishif=0
   nwave = 0
   wave = [CU_WAVE,0.,0.]
   ratio = [1.0,0.5,0.2]
!
!  Global variables
   filetypepow = FTYPE_POW
   modify_range = .false.
   ifRhombAx = 0
   numsymb=0
   erreprof=0.0
   itypep = 1
   asym=0.0
   zeroprof=0.0
   thstepcor=0.0  
   percme=0.0
   scal=1.0
   frac=0.0
   ncycle=0
   icycle=0
   nosing=0
!
   iesdirsear = 0
   iesdircell = 0
   iesspace = -1
   iesspacesav = 0
   itrial = 1
   irunextra = 0
   frac = 10.
   iesext = 0
   shiftzero = -1.E-9
!
   fname3ex=' '
   nprof=1
   thmin=-0.01
   thmax=-0.01
   is_indexing = command_is_after(commands,NTREOR_CMD,icomat)>0 .or.         &
                 command_is_after(commands,DICVOL_CMD,icomat)>0 .or.         &
                 command_is_after(commands,MCMAILLE_CMD,icomat)>0
   if (is_indexing) then
       ksearch = 1
   else
       ksearch = 0
   endif
   nsyn=0
   kalpha= 0
   noghost=0
   istog=3
   itry=0
   fileh=trim(StructureName)//'.pow'
!--
   neutro=RX_SOURCE
   okrad = .false.
   kequirip=0
   neuwei=0
   iprin=0
   scaledata = .false.
   scaleval = 1000.
!corr   alambda=-1.0
   pfwhm=0.1
   kshif(:)=0
   bt=0.0
   sc=-99.9
   knw=0
   nstr=0
   nzro=166
   th=2.0
   rhomin=0.0
   en=1.0
   er=0.3
   npseud=0
   llpse=0
   lix1=0
   lix2=0
   lix3=0
   lix7=0
!
   icmax=icomq(icomat,2)
   ipdir = get_ipdir(commands,icomat)  ! get pointer to first dir in card
!
   loop_datadir: do ic=1,icmax
      ipdir=ipdir+1
      line=card(ipdir)
      diret=line
      call cutst(line,lenp,dire,lend)
      call lcase(dire)
!-- directives with alphanumeric parameters
!-- global directives (extra,sirpow)
      select case (dire(1:4))
        case ('cont')
          call cryst(1)%set_scattering(line)
          if (cryst(1)%numelem() == 0) go to 8000
          call cryst(1)%get_scattering(celem)
          iesdircont = 1
          lix3=2

        case ('isot')
          nistp=nistp+1
          call cutst(line,lenp,dire,lend)
          call lcase(dire)
          isotope(nistp) = trim(dire)
          read(dire(1:2),'(a2)') kistp(nistp)
          !iopt=0
          !call getnum(line,vet,ivet,iv,iopt)
          !if (iopt.eq.-1) go to 8000
          call getnum(line,vet,ivet,iv)
          if (err_string) go to 8000
          if (iv.ne.1) go to 8000
          vistp(nistp)=vet(1)
          lix3=2

        case ('equi')
          kequirip=1

        case ('know')
          knw=1
          if (lenp.gt.0) read(line,'(a)') fname(FC_FILE)

        case ('neut')
          neutro=NEUTRON_SOURCE
          okrad = .true.

        case ('elec')
          neutro=ELECTRON_SOURCE
          okrad = .true.

        case ('spac')
          if (lenp.le.1) go to 8000
!!!TODO:  use set_spg(line,err) 
          spacegr = init_spaceg_type(line) ! set space group in structure spacegr
          call cryst(1)%set_spg(spacegr)
          call lcase(line)
          call StandardSpg(line,ier_spg)
          if (ier_spg /= 0 .and. spacegr%undef()) then
              ier = ier_spg
              go to 8000
          endif
          !if(ier.lt.0)go to 8000
          lix2=1
          iesspace = 0

        case ('form')   ! obsolete directive

!--  sirpow directives
        case ('refl','ref2') ! lettura file h k l f^2 sigma
          if (active_file == REFL_FILE) then
              read(line,'(a)') fileh
          endif

        case ('fobs')        ! file di rifllessi con f e non f^2
          if (active_file == REFL_FILE) then
          endif
!
!--  extra directives
!
        case('file')
          if (active_file == POW_FILE) then
              filetypepow = filetype_from_string(line)
              if (filetypepow == 0) filetypepow = FTYPE_POW
          endif

        case ('patt')
          if (active_file == POW_FILE) then
              if (lenp.eq.0) go to 8000
              read(line,'(a)') fileh
              filetypepow = filetype_from_filename(fileh) 
              if (filetypepow == 0) filetypepow = FTYPE_POW
          endif

        case ('broo')
          if (active_file == POW_FILE) then
              nsyn=1
          endif

        case ('alph') 
          if (active_file == POW_FILE) then
              kalpha=1
          endif

        case ('nogh')
          if (active_file == POW_FILE) then
              noghost=1
          endif

        case ('sync') 
          if (active_file == POW_FILE) then
              nsyn=1
          endif

        case ('peak') ! direttiva obsoleta
          if (active_file == POW_FILE) then
              ksearch = 1
          endif

        case ('exte') 
          if (active_file == POW_FILE) then
              if (lenp.eq.0) go to 8000
              read(line,'(a)') fname3ex
              iesext = 1
!             azzera peak search
              ksearch=0
          endif

        case ('pear')
          if (active_file == POW_FILE) then
              nprof=1
          endif

        case ('gaus')
          if (active_file == POW_FILE) then
              nprof=2
          endif

        case ('2-th')
          if (active_file == POW_FILE) then
              itypep=2
          endif

        case ('nore')
          if (active_file == POW_FILE) then
              iesnorefine = 1
          endif

        case ('find') 
          if (active_file == POW_FILE .and. .not.is_indexing) then ! next command is not indexing
          !if (active_file == POW_FILE) then ! next command is not indexing
              iesspace=1
              iesspacesav = 1
          endif
!
!-- directives with numeric parameters

        case ('sear')
          if (active_file == POW_FILE) then
              call getnum(line,vet,ivet,iv)
              if (err_string) go to 8000
              ksearch=1
              iesdirsear = 1
              if (iv == 0) go to 8000
              if (vet(1) > 0) pkcond%minp = vet(1)
              if (vet(2) > 0) pkcond%maxp = vet(2)
              if (ivet(3) > 0) pkcond%maxpk = ivet(3)
              if (vet(4) > 0) pkcond%amp = vet(4)
              if (ivet(3) < 0 .and. vet(4) > 0) pkcond%maxpk = -1 ! no limit for number of peaks
          endif

        case ('rang')
          if (active_file == POW_FILE) then
              call getnum(line,vet,ivet,iv)
              if (err_string) go to 8000
              select case (iv) 
                 case (2,3)  ! 2)thmin thmax 3) thmin thmax thstep
                   thmin=vet(1)
                   thmax=vet(2)
                   minrange = thmin
                   maxrange = thmax
                 case default
                   go to 8000
              end select
              modify_range = .true.
          endif

        case ('step')
          if (active_file == POW_FILE) then
              call getnum(line,vet,ivet,iv)
              if (err_string) go to 8000
              if (iv /= 1) go to 8000
              if (vet(1) > 0.0) then
                  thstep = vet(1)
                  modify_step = .true.
              else
                  ier = -1
                  go to 8000
              endif
          endif

        case ('hist')
          if (active_file == POW_FILE) then
              call getnum(line,vet,ivet,iv)
              if (err_string) go to 8000
              kistog=1
              if (iv.ne.1) go to 8000
              istog=ivet(1)
              if(istog.ne.3.and.istog.ne.5.and.istog.ne.7)then
                 ier=-1
                 write(Msg,8100) diret(1:Lung(diret))
        8100        format('Error in DATA directive: (histogram bar number can be 3 or 5 or 7)',a)
                 call MsgWinErr('ERROR',Msg,ERR_WINDOW,nExitCode)        
                 return
              endif
          endif

        case ('smoo')
          if (active_file == POW_FILE) then
              call getnum(line,vet,ivet,iv)
              if (err_string) go to 8000
              ismoo=1
              select case (iv)
                 case (1)
                   nsmoo = ivet(1)
                 case (2)
                   nsmoo = ivet(1)
                   npsmoo = ivet(2)
              end select
              if (npsmoo > 2*nsmoo) then
                  ier=-1
                  Msg = "Error in DATA directive "//trim(diret)//': '//  &
              "poly. degree must be <- 2*inter. smoothing"
                  call MsgWinErr('ERROR',Msg,ERR_WINDOW,nExitCode)
                  return
              endif
          endif

        case ('zero')
          if (active_file == POW_FILE) then
              call getnum(line,vet,ivet,iv)
              if (err_string) go to 8000
              if (iv.ne.1) go to 8000
              shiftzero=vet(1)
          endif

        case ('thco')
          if (active_file == POW_FILE) then
              call getnum(line,vet,ivet,iv)
              if (err_string) go to 8000
              if (iv.ne.1) go to 8000
              thstepcor=vet(1)
          endif
!
!-- global directives (extra,sirpow)
        case ('fwhm')
          call getnum(line,vet,ivet,iv)
          if (err_string) go to 8000
          if (iv.ne.1) go to 8000
          pfwhm=vet(1)
          if(pfwhm.lt.0.0.or.pfwhm.gt.1.0) go to 8000

        case ('rhom')
          call getnum(line,vet,ivet,iv)
          if (err_string) go to 8000
          if (iv.ne.1) go to 8000
          th=vet(1)
          if (th.lt.0.0.or.th.gt.1.0) go to 8000

        case ('wave')     ! wave1 wave2 [ratio2]  or wave1 wave2 ratio2 wave3 [ratio3]  or ....
          call getnum(line,vet,ivet,iv)
          if (iv == 0) go to 8000
          nwave = iv / 2 + 1   ! number of values for wavelengths
          wave(1) = vet(1)
          wave(2:nwave) = vet(2:iv:2)
          nrt = (iv-1) / 2     ! number of ratios, last might be omitted
          ratio(2:nrt+1) = vet(3:iv:2)

        case ('cell')
          call getnum(line,vet,ivet,iv)
          if (err_string) go to 8000
          if (iv.ne.6) go to 8000
          do i=1,6
             celltemp(i)=vet(i)
             cell(i)=vet(i)
          enddo
          do i=7,13
             celltemp(i)=0.0
          enddo
          iesdircell = 1
          lix1=1
!corr          call frm01( (/(cell(i),i=1,6),(0.0,i=7,13)/) )
          call cryst(1)%set_unit_cell(set_cell_type(cell))
          cryst(1)%is_symm_user = .true.

        case ('prin')
          call getnum(line,vet,ivet,iv)
          if (err_string) go to 8000
          if (iv.ne.1) go to 8000
          iprin=ivet(1)

        case ('shif')
          call getnum(line,vet,ivet,iv)
          if (err_string) go to 8000
          if (iv.ne.3) go to 8000
          ishif=1
          do i=1,3
             ksh=ivet(i)
             if (ksh.ne.0) then
                 if (mod(24,ksh).ne.0) then
                     write(lo,230)
  230                format(///' origin shift components must be integral multiples of 1/24 ')
                     go to 8000
                 endif
             endif
             kshif(i)=ksh
          enddo

        case ('scal')  
          call getnum(line,vet,ivet,iv)
          if (err_string) go to 8000
          if (iv > 0) then
              scaleval = vet(1)
          endif
          scaledata = .true.

        case default
          write(Msg,6000) diret(1:Lung(diret))
 6000     format('Wrong directive in DATA modulus: ',a)
          ier=-1
          call MsgWinErr('ERROR',Msg,ERR_WINDOW,nExitCode)        
          return

      end select
   enddo loop_datadir

   if (neutro /= RX_SOURCE .and. nsyn == 1) then
       write(Msg,8216)
 8216 format('Ambiguous radiation (neutron/syncrothron)')
       ier=-1
       call MsgWinErr('ERROR',Msg,ERR_WINDOW,nExitCode)        
       return
   endif
   if (kalpha.eq.0.and.kistog.eq.1.and.active_file == POW_FILE)then
       ier=-1
       call MsgWinErr('ERROR',                                           &
       'Alpha2 stripping request is not in input while the number '//    &
       'of bars of the histogram for the alpha2 correction is given',ERR_WINDOW,nExitCode)        
       return
   endif
!
!  Symm. info are not required with  %crystal and dataset after first
   symm_required = is_symm_required(commands) .and. ndataset(dataset) == 0
!
!  Cell 
   if (lix1 <= 0) then
!
!     error message if search or filepeak or peak directives are not used
      if (ksearch == 0 .and. iesext == 0 .and. symm_required) then
          ier=-1
          call MsgWinErr('ERROR','No cell parameters supplied',ERR_WINDOW,nExitCode)        
          return
      endif
   else
      call CheckRhombAx(nSpaceG, celltemp, ifRhombAx)
      call frm01( (/(celltemp(i),i=1,6),(0.0,i=7,13)/) )
      call xfl01
   endif
!
!  Spacegroup
   if (lix2.gt.0.and.iesspace.gt.0) then
       call MsgWinErr('WARNING',                                        &
       'Directives regarding space group are in clash'//char(10)//      &
       'Do you want to perform the space group determination?',         &
       QUEST_WINDOW_YES_NO,nExitCode)        
       if (nExitCode == 2) then
           iesspace = 0
           iesspacesav = 0
       endif
   endif
   if (lix2 == 0) then
       if (symm_required) then
!--- error message if search or filepeak or peak directives are not used
           if (ksearch.eq.0.and.iesext.eq.0.and.iesspace.eq.-1) then
               call MsgWinErr('WARNING',                                &
               'No space group symbol supplied.'//char(10)//            &
               'Space group determination will be performed.',          &
               WARN_WINDOW,nExitCode)        
                iesspace=1
                iesspacesav = 1
           endif
      endif
   else
      if (ier_spg == 0) then
          call insym(ishif,kshif,ier)
          if (ier < 0) then
              return
          endif
      else
          call set_spaceg_new(spacegr)
          call cryst(1)%set_spg(spacegr)
      endif
      call xspinb
      call yfrm02
   endif
! 
   if (lix3 <= 0) then
!--- error message if search or filepeak or peak directives are not used
       if (ksearch.eq.0.and.iesext.eq.0) then
!
!          Controlla se il contenuto e' necessario
           if (.not.next_command_is(commands,icomat,CRYSTAL_CMD)) then  ! jump check with %crystal
               if (is_content_required(commands)) then
                   ier=-1
                   call MsgWinErr('ERROR','No cell content supplied',ERR_WINDOW,nExitCode) 
                   return
               endif
           endif
       endif
   elseif (lix3 == 2) then
       call ordina_elements(celem,neutro)
       call cryst(1)%set_scattering(celem)
   endif
   call xdump
   if (neutro /= NEUTRON_SOURCE .and. nistp > 0) then
       ier=-1
       call MsgWinErr('ERROR','The ISOTOPE directive must be used with the NEUTRON directive',ERR_WINDOW,nExitCode)
       return
   endif
   if (iesdircont.eq.1) then
       if (neutro == NEUTRON_SOURCE) then
          if (nistp.gt.0) then
              do j=1,nistp
                 kpos = string_locate(isotope(j),celem%lab)
                 if (kpos > 0) then
                     celem(kpos)%fact = vistp(j)
                     cycle
                 endif
                 write(Msg,9007) isotope(j)
 9007            format('Atom type ',a,' not expected. Check isotope directive')
                 ier=-1
                 call MsgWinErr('ERROR',Msg,ERR_WINDOW,nExitCode)
                 return
             enddo
          endif
          call elem_set_radiation(celem,neutro)
          call ordina_elements(celem,neutro)
          call cryst(1)%set_scattering(celem)
       endif
       if (neutro == ELECTRON_SOURCE) then
           call elem_set_radiation(celem,neutro)
           call cryst(1)%set_scattering(celem)
       endif
!corr       call print_elements(celem,lo,neutro)
       s3s2 = s3s2_value(celem,neutro)
       if (abs(s3s2) < 0.0001) then
           ier=-1
           write(Msg,'("Sigma(3) / sigma(2)**1.5 = ",f10.3)') s3s2
           call MsgWinErr('ERROR',Msg,ERR_WINDOW,nExitCode)        
           return
       endif
   endif
!
!  Print symmetry and content
   if (mpi_prog_rank == mpi_prog_master) then
       if (lix1 > 0) call print_cell(cell,kpr=lo)
       if (lix2 > 0) call WriteSymmInfo(lo)
       if (numelem(celem) > 0) call print_elements(celem,lo,neutro)
   endif
!
   if (iesnorefine.eq.1.and.active_file == POW_FILE) pkcond%refine = .false.
!
   return

 8000 continue
   ier=-1
   call MsgWinErr('ERROR','Error in DATA directive : '//trim(diret),ERR_WINDOW,nExitCode)
   return

   end

!------------------------------------------------------------------

   subroutine open_diffraction_patt(filnam, ier)
   USE fileutil
   USE General, only: StructureName, fname, ext, INP_FILE, OUT_FILE     !!!!!, alambda
   USE datasetmod
   USE datautil
   USE molcom, only:ifProject
   USE strutil
   USE General, only:lo
   USE variables, only: dataset,cryst
   USE plotstyle, only:styledata
   implicit none
   character*(*), intent(in) :: filnam
   integer                   :: ier
   integer                   :: iflag
   integer                   :: itype, iform, nlenn, jop, jopen, ier2
   integer                   :: i
   type(dataset_type)        :: datas
!
   call load_datafile(datas,filnam,.true.,ier) 
   if (ier == 0) then
       if (ifProject.ne.0) then   ! importante se esegui run successivi
           iflag = 2
!          comment the next line to add pattern
           call chiudi_bin(iflag) ! call InitExpo
       endif
       StructureName = file_rem_ext(file_get_name(filnam))
       call cryst(1)%set_name(StructureName)
       do i=1,size(fname)
          if (i /= INP_FILE .and. i /= OUT_FILE) then
              fname(i) = s_blank_delete(trim(StructureName)//ext(i))
          endif
       enddo
       fname(INP_FILE) = trim(filnam)
       fname(OUT_FILE) = trim(file_rem_ext(filnam))//ext(OUT_FILE)
!
       close(lo,iostat=ier)
       itype = 0
       iform = 1
       nlenn = 80
       jop = jopen(lo,fname(OUT_FILE),itype,iform,ier2)
!
       call clear_dataset(dataset)
       datas%style = styledata%get(1)
       call push_back_dataset(dataset,datas)
       call work_on_pattern(dataset(1),filnam)
!
   endif
!
   return
   end

!---------------------------------------------------------------------------

   subroutine open_pattern(filnam,wavel,ier)
   USE datasetmod
   USE datautil
   USE General, only: StructureName
   USE fileutil, only: file_get_name, file_rem_ext
   USE variables, only: dataset,cryst
   USE molcom, only: jscreen
   USE plotstyle, only:styledata
   implicit none
   character(len=*), intent(in) :: filnam
   real, intent(in)             :: wavel
   integer, intent(out)         :: ier
!corr   integer                      :: ndat
   type(dataset_type)           :: datas
!
!corr   call resize_dataset(dataset,1)
!corr   ndat = ndataset(dataset)
!corr   if (wavel > 0.0) call dataset(ndat)%set_wave(1,[wavel])
!corr   call load_datafile(dataset(ndat),filnam,jscreen>0,ier)
!corr   if (ier == 0) then
!corr       StructureName = file_rem_ext(file_get_name(filnam))
!corr       call cryst(1)%set_name(StructureName)
!corr       call work_on_pattern(dataset(ndat),filnam)
!corr   endif
   if (wavel > 0.0) call datas%set_wave(1,[wavel])
   call load_datafile(datas,filnam,jscreen>0,ier) 
   if (ier == 0) then
       StructureName = file_rem_ext(file_get_name(filnam))
       call cryst(1)%set_name(StructureName)
       datas%style = styledata%get()
       call clear_dataset(dataset)
       call push_back_dataset(dataset,datas)
       call work_on_pattern(dataset(1),filnam)
   endif
!
   end subroutine open_pattern

!---------------------------------------------------------------------------

   subroutine work_on_pattern(datas,filnam)
   USE Conteggi, only: npunti
   USE view, only: vedinew
   USE messagemod
   USE enable_amb
   USE datasetmod, only: dataset_type
   USE molcom, only: jscreen
   implicit none
   type(dataset_type), intent(in) :: datas
   character(len=*), intent(in)   :: filnam
!
   call dataset_to_expo(datas)

   if (jscreen > 0) then
       call AbilitaPaned(1)
       call init_messages()
       call write_message_pattern(filnam)
       call vedinew(8,1,npunti)
       call abilita_tasti('pattern')
              call wmenu_set_state('Range'//char(0),1)
       call SetWindowTitle(filnam)
!corr       call abilita_tasti('structure')
       call toolbar_set_state('continue'//char(0),2)
   endif

   call init_dati()
!
!  create background FIXME-non serve se fai una open_background
!   call fillcounts()
!   call back_for_peaksearch()
!!!!!test peak search
   !     call test_peaksearch()
!!!!!test peak search

!!!!!!!!!!!!!!!prova index
      !call run_indexing(filehpow, alambda, 'mcmaille')
      !call run_indexation(filehpow, alambda, 'dicvol')
      !call run_indexation(filehpow, alambda, 'ntreor')
!!!!!!!!!!!!!!!prova index

   end subroutine work_on_pattern
!---------------------------------------------------------------------------

!   subroutine test_peaksearch()
!   USE peak_mod
!   USE messagemod
!   USE Conteggi
!   USE view
!   USE proginterface, only:fillcounts
!!corr   USE indexing
!   integer :: ier
!   common /ksearch/ ksearch
!   integer :: ksearch
!   call fillcounts()
!   call back_for_peaksearch()
!   !call peaksearch(shiftzero,ier)
!   ksearch = 1
!   call peak1(ier)
!   call vedinew(5,1,npunti)
!   call write_message('Number of peaks: ',inum=npk,pos=2)
!   end subroutine test_peaksearch

!---------------------------------------------------------------------------
   
   subroutine dataset_to_expo(datas)
   USE datasetmod, only: dataset_type
   USE General, only:alambda, neutro, nsyn, wavel
   USE patternref, only:filehpow
   USE arrayutil
   USE indexing
   implicit none
   type(dataset_type), intent(in) :: datas
!
   if (size_array(datas%x) > 0) then
       call dataset_to_thetaint(datas)
       alambda = datas%wave(1)
       wavel = datas%wave(1)
       call init_indexation(alambda)
       filehpow = trim(datas%fname)
       neutro = datas%radtype
       if (datas%sync) then
           nsyn = 1
       else
           nsyn = 0
       endif
   endif
   end subroutine dataset_to_expo

!---------------------------------------------------------------------------

   subroutine dataset_to_thetaint(dataset)
   USE Conteggi
   USE PatternRef
   USE arrayutil
   USE datasetmod, only: dataset_type
   implicit none
   type(dataset_type), intent(in) :: dataset
!
   call AllocaConteggi(2)
   npunti = size_array(dataset%x)
   call AllocaConteggi(1)
   theta_int(:,1) = dataset%x
   theta_int(:,2) = dataset%y
   theta_int(:,7) = theta_int(:,1)
   theta_int(:,3:6) = 0.0
   thstep = dataset%tstep
   thmin = dataset%tmin
   thmax = dataset%tmax
!
   end subroutine dataset_to_thetaint


!---------------------------------------------------------------------------

   subroutine GenInfoV2
   USE General, only: iscra,slopeWils,bt
   USE PhaseModule, only: numset,nset,erre
   USE atom_type_util
   USE stat
   USE variables, only: cryst,dataset
   USE crystal_phase
   USE datasetmod
   use prog_constants, only: DEF_WAVE
!
   implicit none
   real                                          :: fom,cfom,ep,dens,wave
   integer                                       :: lsn,lfw,llw,lpb,ll16,ipos,iphb
   integer                                       :: i,j,nph,z_val,radtype
   character(len=:), allocatable                 :: strf
   type(element_type), dimension(:), allocatable :: chform
   dimension ipos(302),cfom(302),fom(11),ep(2000),iphb(2000)
!
   if (ndataset(dataset) > 0) then
       call dataset(1)%prn(iscra,.true.)
       radtype = dataset(1)%radtype
       wave = dataset(1)%wave(1)
   else
       wave = DEF_WAVE
       write(iscra,'(" Wavelength =",f10.6/)') wave
       radtype = RX_SOURCE
   endif
!
   do nph=1,numphase(cryst)
      if (numphase(cryst) > 1) write(iscra,'(/,100("=")/)')
      call cryst(nph)%print(iscra,'NCSE',radtype=radtype)
!
      if (numphase(cryst) == 1) then
          if(abs(slopeWils).gt.0.0) then
              write(iscra,'(" Wilson Thermal Factor B(iso) = ",f7.3)') bt
              write(iscra,*)
          endif
!
          if (numset > 0) then
              call srlin(16,lsn,lfw,llw,lpb,ll16)
              if (lfw.gt.0) then
                  write(iscra,'("                        Figures of Merit")')
                  write(iscra,'("   Set  (trial)",1x,"  mabs  alcomb R(x,y)"," cphase  cfom",3x,"nund")')
                  call ysfl16(cfom,ipos,nset)
                  do j=1,nset
                     i = ipos(j)
                     call snr16(i,fom,ep,iphb)
                     write(iscra,'(i6,"  (",i5,")",5f7.3,i6)') j,nint(fom(10)),(fom(i),i=1,5),nint(fom(6))
                  enddo
              endif
              write(iscra,*)
          endif
      endif
!
      if (cryst(nph)%natoms() > 0) then  
          write(iscra,'(" Nr. of atoms in the asymmetric unit: ",i0)')cryst(nph)%natoms()
          if (cryst(nph)%numref() > 0) then
              call cryst(nph)%sfactor()
              if (cryst(nph)%is_known()) then
                  erre = Rvalue(cryst(nph)%ref%fv,cryst(nph)%ref%fc)
              else
                  erre = Rvalue(cryst(nph)%ref%fo,cryst(nph)%ref%fc)
              endif
              write(iscra,'(" R-f Fit: ",f0.3)') erre
          endif
          call print_atoms(cryst(nph)%at,height=.true.,kpr=iscra)
          z_val = Z_value(cryst(nph)%at,cryst(nph)%spg%nsymop,chform)
          call chemical_formula(chform,strf,fform=2,ord=.true.,hide1=.true.,hidecharge=.true.)
          write(iscra,'(" Formula:         ",a          )')strf
          write(iscra,'(" Z:               ",i0         )')z_val
          write(iscra,'(" Volume per atom: ",f0.3," °A3")')  &
                volume_per_atom(cryst(nph)%at,cryst(nph)%spg%nsymop,cryst(nph)%cell%volume())
          dens = cryst(nph)%density()
          write(iscra,'(" Density:         ",f0.3," g/cm3")')dens
          write(iscra,'(" Mu:              ",f0.3," cm-1")')linear_abs_coeff(cryst(nph)%at,dens,wave)
      endif
   enddo
!
   end subroutine GenInfoV2

! -----------------------------------------------------------------

   subroutine run_indexing(sprog)
   USE indexing
   USE timer
   USE general, only: lo
   USE progtype, only: DISTRIBUTED_VER
   implicit none
   character(len=*), intent(in)                    :: sprog
   type(cell_info_type), dimension(:), allocatable :: pcell
   integer                                         :: ier
   logical                                         :: test = .true.
   type(timer_type)                                :: timeind
!   
   call timeind%set_as_elapsed()
   call timeind%tic()
   select case (sprog)
      case ('dicvol')
        call timeind%set_name(PINDEX_NAME(DICVOL06_PROG))
        call run_dicvol(indx_set(DICVOL06_PROG),pcell)
        call add_to_cell_list(cell_list,pcell)
      case ('mcmaille')
        call timeind%set_name(PINDEX_NAME(MCMAILLE_PROG))
        call run_mcmaille(indx_set(MCMAILLE_PROG),pcell)
        call add_to_cell_list(cell_list,pcell)
      case default
        !call ntreor(.false.,ier)
        !    write(0,*)'NTTREOR run'
        call timeind%set_name(PINDEX_NAME(NTREOR09_PROG))
        call run_ntreor(.false.,ier)
   end select
   call timeind%toc()
   call timeind%tprint(lo)
   call timeind%tprint(0)
!
   if (test .and. .not.DISTRIBUTED_VER) call test_indexing(cell_list)
!    
   end subroutine run_indexing

! -----------------------------------------------------------------

   subroutine save_cell_list(filnam)
!
!  Save cell list in a exp or txt file
!
   USE indexing
   USE fileutil
   USE strutil
   USE programinput
   implicit none
   character (len=*) :: filnam
   type(file_handle) :: cf
!
   filnam = remc0(filnam)
   if (s_eqi(get_extension(filnam),'exp')) then
       call write_input_file(input_file_create(filnam),'space',cell_list) 
   else
       call cf%fopen(filnam,'w')
       call print_cell_list(cell_list,cf%handle())
       call cf%fclose()
   endif
!
   end subroutine save_cell_list


   subroutine run_peaksearch()
   USE Conteggi, only: npunti, theta_int
   USE counts, only:deltatval,deltadval,delta_from_lambda
   USE peak_mod
   USE proginterface, only:fillcounts
   USE General, only:alambda
   USE variables, only: dataset
   USE arrayutil
   !use filereading
   implicit none
   real, dimension(:), allocatable :: yc
   real, parameter                 :: SRANGE = 0.15 ! 2theta-range for peak_search at 1.54056
!
!  Subtract background
   if (size_array(dataset(1)%yb) /= npunti) then
       !call dataset_to_thetaint(dataset(1))
       call back_for_peaksearch(.true.)
   endif
   allocate(yc(npunti))
   !where (backgr(:) > theta_int(:,2)) 
   !       yc(:) = 0
   !elsewhere
           yc(:) = theta_int(:,2) - dataset(1)%yb(:)
   !endwhere
!
!  compute srange
   !pkcond%srange = init_peak_range(theta_int(:,1),theta_int(:,2),alambda)
   pkcond%srange = init_peak_range(theta_int(:,1),yc,alambda)
   !write(0,*)'SRANGE =',pkcond%srange
!
   call peak_create(theta_int(:,1),yc,pkind,pkcond,alambda,pkindtot)
   pkcond%maxpk = -1 ! unset max number of peaks for GUI
   pkcond%minpk = -1 ! unset min number of peaks for GUI
!
   call update_peak_graph()
   !call write_column('picchi.txt',xcol1=pkind%getx(),metad='#2theta values')
!
   end subroutine run_peaksearch

! ---------------------------------------------------------------------

   subroutine run_peaksearchwin()
!
!  When the user press the button 'Pattern' > 'Peak search'
!
   USE patternref, only: thmin,thmax
   USE peak_mod
!
!  Reset the range to create the array pkindtot
   pkcond%minp = thmin
   pkcond%maxp = thmax
!
   call run_peaksearch()
   end subroutine run_peaksearchwin

! ---------------------------------------------------------------------

   subroutine update_peak_graph()
   USE molcom, only: jscreen
   USE enable_amb
   USE messagemod
   !USE peak_util
   USE peak_mod
   USE view
!
   call write_message('Number of peaks: ',inum=numpeaks(pkind),pos=2)
!
   if (jscreen > 0) then
       call update_peak_list()
       call abilita_tasti('peaks',state=1)
       call vedinew(5)   !!!!,1,npunti)
   endif
!
   end subroutine update_peak_graph

! ---------------------------------------------------------------------

   subroutine search_peaks(iAction,statomin,statomax,search_value)
   USE VIEW
   USE General, only: alambda
   USE PatternRef
   USE peak_mod
   USE messagemod
   USE conteggi
!corr   USE counts
   USE variables, only: dataset
!
   implicit none
   integer                                          :: iAction
   real                                             :: statomin,statomax
   real, dimension(5)                               :: search_value
   type(peak_type), dimension(:), allocatable, save :: pksav
   type(peaks_condition_type), save                 :: pkconds
   type(peaks_condition_type)                       :: pkcond_def
   real                                             :: scaln
   integer :: npeak
!
   select case (iAction)
      case (0)    ! Cancel (reset the conditions)
        pkcond = pkconds
        call peak_copy(pkind,pksav)
        call write_message('Number of peaks: ',inum=numpeaks(pkind),pos=2)
        call vedinew()

      case (1)    ! Open window, pass variables to graphic, save settings
        if (.not.allocated(pkindtot)) then  
            if (.not.allocated(pkind)) then ! Peak search has been not performed
                call run_peaksearch()
            else                            ! Peaks from external file: create only array pkindtot
                pkcond%srange = init_peak_range(theta_int(:,1),theta_int(:,2),alambda)
                call peak_create(theta_int(:,1),theta_int(:,2)-dataset(1)%yb(:),pksav,pkcond,alambda,pkindtot)
            endif
        endif
        pkconds = pkcond
        call peak_copy(pksav,pkind)
        statomin = thmin
        statomax = thmax
        if (pkcond%minp > 0) then
            search_value(1) = pkcond%minp
        else
            search_value(1) = thmin
        endif
        if (pkcond%maxp > 0) then
            search_value(2) = pkcond%maxp
        else
            search_value(2) = thmax
        endif
        search_value(3) = pkcond%amp
        search_value(4) = numpeaks(pkind)
        search_value(5) = numpeaks(pkindtot)

      case (2:6)   ! Performe new peak search
!
        if (iAction == 6) then ! Default required by user
            search_value(1) = max(pkcond_def%minp,thmin)
            search_value(2) = max(pkcond_def%maxp,thmax)
            search_value(3) = pkcond_def%amp
        endif
        pkcond%minp = search_value(1)
        pkcond%maxp = search_value(2)
        pkcond%amp = search_value(3)
        if (numpeaks(pkindtot) == 0) return

        select case (iAction)
          case (2,3,4,6)        ! range (2,3), threshold (4),  default(6)
            call peak_copy(pkind,pkindtot)
            call filtrasog(pkind,pkcond%amp/100,npeak,code=1)
            call peak_select(pkind,pkcond%minp,pkcond%maxp,DELETE_PEAK,outside=.true.)
            call peak_delete_selected(pkind)
            call peak_sort(pkind,ORD_BY_X)
!corr            if (numpeaks(pkind) > 0) call set_intensities(pkind,theta_int(:,7),theta_int(:,2),backgr)
            search_value(4) = numpeaks(pkind)   ! force update of num. of peaks
            call update_peak_graph()

          case (5)            ! number of peaks was changed
            pkcond%maxpk = int(search_value(4))
            if (pkcond%maxpk == numpeaks(pkind)) return
            pkcond%amp = 0  
!
!           Recover peaks and intensities for compute new amp (necessary also if maxpk < numpeaks)
            call peak_copy(pkind,pkindtot)
            call filtrasog(pkind,pkcond%amp/100,npeak,code=1)
            call peak_select(pkind,pkcond%minp,pkcond%maxp,DELETE_PEAK,outside=.true.)
            call peak_delete_selected(pkind)
!
!           Select nmaxpk peaks for intensities
            call peak_sort(pkind,-ORD_BY_Y)
            scaln = 100 / maxval(pkindtot%gety())             ! recompute scale on all peaks
            if (numpeaks(pkind) > pkcond%maxpk) then
                pkcond%amp = pkind(pkcond%maxpk+1)%gety()*scaln   ! define new threshold
                search_value(3) = pkcond%amp                 ! force update of num. of peaks
                call reallocate_peaks(pkind,pkcond%maxpk)
            else   ! too peaks required!
                pkcond%amp = pkind(numpeaks(pkind))%gety()*scaln   ! define new threshold
                search_value(3) = pkcond%amp        ! force update of amplitude
                search_value(4) = numpeaks(pkind)   ! force update of num. of peaks
            endif
            call peak_sort(pkind,ORD_BY_X)
!corr            if (numpeaks(pkind) > 0) call set_intensities(pkind,theta_int(:,7),theta_int(:,2),backgr)
            call update_peak_graph()
         
        end select

   end select

   end subroutine search_peaks

! ---------------------------------------------------------------------

   subroutine cell_dialog(ier)
   USE PatternRef
   USE General, only: POW_FILE,ifAutomatic,lo,cell
   USE enable_amb
   USE commandsmod
   USE comandiexpo, only:icomat
   USE strutil, only:cutst,remc0,addc0
   USE sinterface
   USE molcom, only: jscreen
   USE indexing
   USE spginfom
   USE unit_cell
!
   implicit none
   integer, intent(out) :: ier
   common/spaceindexing /ispaceindexing,fspaceindex(660),    &
                      ntotmult1,ntotmult2,percind(660),errespac(660),lauecell(660)
   integer :: ispaceindexing,ntotmult1,ntotmult2
   real    :: percind,errespac,fspaceindex
   character(len=20) :: lauecell
   common/iesext/iesext
   integer :: iesext
   common /ksearch/ ksearch
   integer :: ksearch
   common /definespac/iesspacesav,itrial,irunextra,nlin
   integer :: iesspacesav,itrial,irunextra,nlin
   common /spacegroup/iesspace,shiftzero
   integer :: iesspace
   real :: shiftzero
   common /centering/CellType,CellAxis,nruntre,iesdircell
   integer :: nruntre,iesdircell
   character CellType,CellAxis*3
   common /prova_kalpha/ kalphasav
   integer :: kalphasav
   integer :: kfindspace
   character(len=:), allocatable  :: line
   integer :: ncell, icell
   type(spaceg_type) :: spgtemp
!
   ier = 0
   call ClearMessage()
!
   kcount_status = 1  ! attiva info riflessi sulla statusbar
   call print_cell_list(cell_list,lo,'List of plausible cell parameters:')
   if (ifAutomatic == 1 .or. jscreen == 0) then
       icell = 1     ! select the first cell
       call viewrefcell(icell)  
   else
       call toolbar_set_state('continue'//char(0),0)
       call abilita_tasti('indexwin',0)
       call abilita_tasti('peaks',0)
       ncell = number_of_cells(cell_list)
       kexitrun = 0
       call init_viewcellref()
       call open_cell_dialog(ncell, cell_list, icell)
       call end_viewcellref()
       call abilita_tasti('indexwin',1)
       call abilita_tasti('peaks',1)
       if (kexitrun /= 0) return  ! exit from File > Exit !!!!FIXME - non funziona se fai Pattern > Indexing
   endif
!
!  Exit for 'cancel' (icell=0); %end with -nogui or -auto; index is last command with -nogui or -auto
   if (icell == 0 .or. ( (jscreen == 0 .or. ifAutomatic == 1) .and.   &
      (next_command_is(commands,icomat,'end') .or. (icomat == ncommands(commands))))) then
       !call abilita_tasti('pattern')
       !call abilita_tasti('peaks',1)   !!!add abilita index
       if (jscreen > 0) call toolbar_set_state('continue'//char(0),2)
!!! reset this variables to avoid problems in data1 for new run with ntreor
!!!FIXME - check is necessary also for ntreor after structure solution
       irunextra = 0
       iesspacesav = 1
       itrial = 1
       kexitrun = 2  ! force exit from loop in gescom
       return
   endif
   if (jscreen > 0) call toolbar_set_state('continue'//char(0),1)
   write(lo,'(//a)')'       The following cell has been selected: '
   call print_cell(cell_list(icell)%par,cell_list(icell)%sd,cell_list(icell)%sys,kpr=lo)
!
   ispaceindexing = -1
   ksearch = 0
   iesdircell = 1
   iesext = 0
   iesspace = 1
   iesspacesav = 1
!
!  Recupera il gruppo spaziale se è presente la direttiva space
   line = get_string_directive(commands,'space')
   kfindspace = 1   ! forza check sul gruppo spaziale
   if (len_trim(line) > 0) then
!
!      Check compatibility between spg in input file and new cell
       spgtemp = init_spaceg_type(line)
       if (csys_from_cell(cell) == spgtemp%csys_code) then
           call set_spaceg(line)  ! spacegr is now defined
           call yfrm02
           kfindspace = 0 
       endif
   endif
!
   call cell_spg_dialog(kfindspace)
   if (kexitrun /= 0) return
!
   if (kfindspace == 1) then
!      findspace+export del file nome1.exp
!      nota che findspace azzera isspace
!      se ho dato peaksearch + findspace non rifaccio correzione kalpha
       kalphasav = 0
       if (kalpha.gt.0) then
           kalpha = 0
           kalphasav = 1
       endif
       call datanew(POW_FILE,.true.,.true.,ier)
!corr       if (kexitrun > 0 .or. ier /= 0) return
   endif
!
   kcount_status = 0
!
   return
   end subroutine cell_dialog

!-----------------------------------------------------------------------

   subroutine cell_spg_dialog(kfindspace)
   USE General, only: cell,celem,ifAutomatic,neutro,POW_FILE
   USE elements
   USE sinterface, only: spaceg_string
   USE commandsmod
   USE molcom, only: jscreen
   USE strutil
   USE PatternRef, only: kexitrun
   USE variables, only: cryst,dataset
   USE crystal_phase, only: get_strelem
   USE errormod
   USE unit_cell
   USE datamod
   implicit none
   integer, intent(inout)                        :: kfindspace
   character(len=18)                             :: spacechar, spacecharold
   real, dimension(6)                            :: cellold
   type(element_type), dimension(:), allocatable :: elem,elemold
   character(len=:), allocatable                 :: cell_cont
   integer                                       :: i,iflagref,kreturn,ier,npq
   character(len=256) :: contentC
   common /definespac/iesspacesav,itrial,irunextra,nlin
   integer :: iesspacesav,itrial,irunextra,nlin
   common /spacegroup/iesspace,shiftzero
   integer :: iesspace
   real :: shiftzero
   common /centering/CellType,CellAxis,nruntre,iesdircell
   character CellType,CellAxis*3
   common/spaceindexing /ispaceindexing,fspaceindex(660),    &
          ntotmult1,ntotmult2,percind(660),errespac(660),lauecell(660)
   integer :: ispaceindexing,ntotmult1,ntotmult2
   real    :: percind,errespac,fspaceindex
   character(len=20) :: lauecell
   integer :: nruntre,iesdircell
   logical :: kgener
   type(error_type) :: err
!
   spacechar = spaceg_string()
!
!  Salva i vecchi valori delle variabili
   cellold(1:6) = cell(1:6)
   spacecharold = spacechar
   call copy_elem(elemold,celem)
!
!  Apri la finestra per modificare cella,gruppo spaziale, contenuto
   spacechar = trim(spacechar) // char(0)
   cell_cont = cryst(1)%get_strelem()
   if (len_trim(cell_cont) == 0) cell_cont = trim(get_string_directive(commands,'cont')) !!!corr //char(0)    !find content in input file
   if (len_trim(cell_cont) == 0) call chemical_formula(celem,cell_cont) !extract from celem if content is not in input file
   if (ifAutomatic == 0 .and. jscreen /= 0) then  !if auto or nogui jump this part
       contentC = cell_cont//char(0)
       call cell_parameters_dlg(cell,kfindspace,spacechar,contentC,kreturn,len(contentC))
       if (kreturn == 0) then
           call toolbar_set_state('continue'//char(0),0)
           kexitrun = 2
           return  ! exit required by user
       endif
       cell_cont = remc0(contentC)
   endif
   iesspace = kfindspace
   spacechar = remc0(spacechar)
   iesspacesav = iesspace
!
!  Convert string in new cell content
   call read_chemformula(cell_cont,ier,elem=elem)
   if (ier > 0 .or. numelem(elem) == 0) then
       kexitrun = 1
       return 
   else
       call elem_set_radiation(elem,neutro)
       call ordina_elements(elem,neutro)
   endif
!
!  update content if it has changed in type or number
   if (.not.equal_elements(elemold,elem)) then
       call copy_elem(celem,elem)
       call cryst(1)%set_scattering(celem)
       call cryst(1)%set_strelem(remc0(cell_cont))
       call WriteContInfo(npq,ier)    !!!FIXME
   endif
!
!  cell has been changed ...
   kgener = .false.
   if (.not.cryst(1)%is_cell()) then
!      cell may be undef. if Import xpd > Pattern > Space Group Determ.
       call frm01( (/(cell(i),i=1,6),(0.0,i=7,13)/) )
       call cryst(1)%set_unit_cell(set_cell_type(cell))
       iesdircell = 1
       kgener = .true.
   else
       if (any(abs(cellold(:) - cell(1:6)) > epsilon(1.0))) then   ! da migliorare
           call frm01( (/(cell(i),i=1,6),(0.0,i=7,13)/) )
           call cryst(1)%set_unit_cell(set_cell_type(cell))
           cryst(1)%is_symm_user = .true.
           iesdircell = 1
           kgener = .true.
       endif
   endif
!
   if (kfindspace == 0) then
!
!      Prendi il gruppo spaziale dell'utente
       call make_new_spg(spacechar,1,err)
       if (err%signal) then
           call err%print()
           kexitrun = 1
           return
       endif
       call yfl02
!
       ispaceindexing = 0
!
       call gener(dataset(1)%wave,iflagref,.true.)
       call datanew(POW_FILE,.true.,.true.,ier) ! rigenera i riflessi
       call export_file_exp()

   else
!
!      spg may be undef. if Import xpd > Pattern > Space Group Determ.
       if (.not.cryst(1)%is_spg()) then
           call gener_findspace(ier)
           kgener = .true.
       endif
   endif
   if (kgener) call gener(dataset(1)%wave,iflagref,.true.)
!
   end subroutine cell_spg_dialog

!-----------------------------------------------------------------------
!corr
!corr   subroutine unpack2(ind,ih,kk,il)
!corr   implicit none
!corr   integer, intent(in)  :: ind
!corr   integer, intent(out) :: ih, kk, il
!corr   integer              :: is
!corr   ih = ind/262144
!corr   is = ind - 262144*ih
!corr   kk = is/512-256
!corr   il = is-512*(kk+256)-256
!corr   return
!corr   end
!corr
!corr!----------------------------------------------------------------------
!corr   real function rhof(p,jh,jk,jl)
!corr   implicit none
!corr   real, dimension(6), intent(in) :: p
!corr   integer, intent(in)            :: jh,jk,jl
!corr   real                           :: rho2
!corr   rho2=p(1)*float(jh*jh)+p(2)*float(jk*jk)   &
!corr    +p(3)*float(jl*jl)+p(4)*float(jh*jk)      &
!corr    +p(5)*float(jh*jl)+p(6)*float(jk*jl)
!corr   rhof=sqrt(rho2)
!corr   end function rhof
!corr
!-----------------------------------------------------------------------

   real function ctheta(rho)
!
!-- compute 2-Theta value from ( sin(theta)/lambda ) ** 2
   USE General, only: alambda
   USE trig_constants
   implicit none
   real, intent(in) :: rho
   real :: theta
   theta=sqrt(rho)*alambda
   if (theta.gt.1.) theta=1.
   theta=asin(theta)*rtod
   ctheta=2.0*theta
   return
   end

!-----------------------------------------------------------------------
   subroutine obsin(kv,ibrav,klatt,ksym,vsym)
!
!  Fill irefvet before call this sub. (call checkeq)
!
   USE Riflessi
   USE General
   USE Symmetry
   USE arrayutil
   USE reflection_type_util, only: extin
   USE mpi_prog
   implicit none
   integer, intent(in) :: klatt, ksym
   integer, intent(in) :: ibrav(6,13),vsym(48)
   integer, intent(in) :: kv(3)
   integer :: indh,indk,indl,inbl,id1,jcode,jex
   integer :: indh1,indk1,indl1,idelt,isgnn
   real    :: rho, ctheta, rhomax1
   integer :: hmin,hmax,kmin,kmax,lmin,lmax                  
   logical :: kpr = .false.
   integer :: nhmax,nkmax,nlmax,nru,nr
!
   nhmax=0
   nkmax=0
   nlmax=0
   rhomax1=rhomax
!
!  at this point the allowed reflections are in memory
!
   nru=0
   nr=nrefl
!--
   call indmm_old(hmin,hmax,kmin,kmax,lmin,lmax)
!--
   nhmax=hmin
   nkmax=kmin
   nlmax=lmin
!--
   do indh=hmin,hmax
      do 700 indk=kmin,kmax
         do 600 indl=lmin,lmax
!---
!---        vedere se il nuovo riflesso generato e' equivalente
!---        ad uno gia' considerato, in questo caso :
!---
            if (indh.eq.0.and.indk.eq.0.and.indl.eq.0) go to 600
!   
!           TRIGONALE : va' considerata la condizione aggiuntiva |h|>|k|
!                      nel caso (+-+)
!                     
            if (jsys.eq.5.and.indk.lt.0.and.iabs(indh).lt.iabs(indk)) goto 600
!   
!           CUBICO :  va' considerata la condizione aggiuntiva h<=k<=l
!    
            if(jsys.eq.6.and.(indk.lt.indh.and.indk.gt.indl)) goto 600
!
            indh1=indh
            indk1=indk
            indl1=indl
            call extin(jex,indh1,indk1,indl1,ibrav,klatt,ksym,vsym,nsym,kmat,tmat,icent)
            if (jex.eq.1) goto 600
            call geneq_old(1,indh1,indk1,indl1,idelt,jcode,isgnn,id1,rho)
            inbl=indh1*kv(1) + indk1*kv(2) + indl1 + kv(3) 
!--         in data il superindice e' calcolato come:
!           inbl= lh*kv(1) + lk*kv(2) + ll + kv(3)
            inbl=iabs(inbl)
            if (inbl > size(irefvet)) go to 600 ! outside experimental range
            if (irefvet(inbl).ne.0) go to 600
            rhomax=rhomax1
            if (rho.gt.rhomax1.or.rho.lt.rhomin) go to 600
! --
! --        ibig e' riempito con il riflesso non osservato
! --
            irefvet(inbl)=1
            nru=nru+1
            if (kpr) then
                write(lo,7040) indh1,indk1,indl1,ctheta(rho)
 7040           format(' not measured reflection :',3i4,'  2-Theta = ',f10.4)
            endif
600    continue
700 continue
   enddo
!
   if (mpi_prog_rank == 0) then
       if (nru.gt.0) write(lo,910) nru
!corr      900 format(10x,i5, '  generated reflections ')
  910  format(/,10x,i5, '  not measured reflections in data')
   endif
   call delete_array(irefvet)
!
   return
   end

!-----------------------------------------------------------------------

   subroutine open_cell_list()
   USE patternref, only: kexitrun
   USE indexing
   USE Conteggi, only: npunti
   USE variables, only: dataset
   implicit none
   integer :: ier
!
   if (number_of_cells(cell_list) == 0) return
!
!  scarica in theta_int
   call dataset_to_thetaint(dataset(1))
   call ysfr37(npunti)  ! for extra
!
   call init_recycle()  ! init for extra in cell_dialog (find space g.) and gescom
   call cell_dialog(ier)
   if (kexitrun > 0) return
   call restart_command()
!
   end subroutine open_cell_list

!---------------------------------------------------------------------

   subroutine restart_command()
   USE General, only: POW_FILE, card
   USE commandsmod
   USE comandiexpo
   USE molcom, only: jscreen
   implicit none
   integer :: iEnd, ier
   integer :: ip
!
   ip = command_position(commands,'data',back=.true.)
   call add_continue(commands,ip)
!
   !call print_commands(commands,0)
   Messages(:) = ' ' ! reset all messagges for sub. scr2
   call create_expo_commands(commands,ipcom,icomq,card)
   icomat = ip
   ipdir = sum(commands(:ip)%ndir)
   if (jscreen > 0) call WaitCursorOn()
   call gescom(POW_FILE,iEnd,ier)
   if (jscreen > 0) call WaitCursorOff()
!
   end subroutine restart_command

!------------------------------------------------------------------

   subroutine data3(active_file,ier)
   USE Riflessi
   USE xdataMod
   USE Conteggi
   USE General
   USE RandomStaff, only: iprandomax,icomatsav,ipdirsav
   USE PatternRef
   USE unit_cell
   USE elements
   USE atom_type_util
   USE mpi_prog
   implicit none
   common /ureq1/ ncycl,nroset,nundset,nsec,jback,uvwt(8),itry,pfwhm,kequirip
   integer :: ncycl,nroset,nundset,nsec,jback,itry,kequirip
   real :: uvwt,pfwhm
   common/sinte/ abc(3),ang(3),astar,bstar,cstar,cosas,cosbs,coscs
   real :: abc,ang,astar,bstar,cstar,cosas,cosbs,coscs
   common /centering/CellType,CellAxis,nruntre,iesdircell
   integer :: iesdircell,nruntre
   character CellType,CellAxis*3
   common /celsav/cellas(6),porig(6)
   real :: cellas,porig
   common /direx/ncycle,icycle,nosing,izero ,zeroprof,asym,scal,frac,noint,ialt,sigma,if92,nocell
   integer :: ncycle,icycle,nosing,izero,noint,ialt,if92,nocell
   real :: zeroprof,asym,scal,frac,sigma
   common/spaceindexing /ispaceindexing,fspaceindex(660),ntotmult1,ntotmult2,percind(660),errespac(660),lauecell(660)
   integer :: ispaceindexing,ntotmult1,ntotmult2
   real :: fspaceindex,percind,errespac
   character*20 lauecell
   integer, intent(in) :: active_file
   integer :: is3s2,natc
   integer :: i,mn
   real :: s3s2p,vol
   integer :: kuser,istop,ier,iy
!
   if (active_file == POW_FILE) then
      do i=1,3
         abc(i)=cell(i)
         ang(i)=cell(i+3)
      enddo
      cellas(1)=abc(1)
      cellas(2)=abc(2)
      cellas(3)=abc(3)
      cellas(4)=ang(1)
      cellas(5)=ang(2)
      cellas(6)=ang(3)
      porig(1)=pp(1)
      porig(2)=pp(2)
      porig(3)=pp(3)
      porig(4)=pp(4)
      porig(5)=pp(5)
      porig(6)=pp(6)
      do i=1,npunti
         theta_int(i,4)=theta_int(i,1)
         theta_int(i,5)=theta_int(i,2)
      enddo
!#ifdef MPI
      if (mpi_prog_rank == 0) then
          call ysfr37(npunti)
          call ysfl37(npunti)
      endif
!#endif
   endif
   anat = 0.0
   natc = nint(natom_cell(celem,neutro==0))
   if(natc > 0 .and. spacegr%nsymop > 0) then
      anat = nasym_unit(celem,spacegr%nsymop,neutro==0)
      s3s2p = s3s2_value(celem,neutro,spacegr%ncoper)
      s3s2=s3s2p
      is3s2=1./(s3s2p*s3s2p)+0.5
   endif
!
!  Print info about cell and chemical content
   if (mpi_prog_rank == 0) then
       if (ispaceindexing==0) then
           if (numelem(celem) > 0 .or. iesdircell == 1) write(lo,*)
           if (numelem(celem) > 0) then
               write(lo,'(19x,a,f0.2)') 'Number of atoms in asymmetric unit = ',anat
               write(lo,'(11x,a,i0)') 'Equivalent number of equal atoms in primitive cell = ',is3s2
           endif
           if (iesdircell == 1) then
               vol = cell_volume(cell)
               if (numelem(celem) > 0) write(lo,'(26x,a,f0.2)') 'Volume per atom = ',volume_per_atom(celem,vol)
               write(lo,'(26x,a,f0.2)') 'Volume of the cell = ',vol
           endif
           if (numelem(celem) > 0 .or. iesdircell == 1) write(lo,*)
       endif
   endif
!  calculate number of reflexions for (sem)invariant searching
   if (nstr.gt.0) goto 400
   mn = int(4.0*anat+100.5)
   !if (icent.eq.1) mn = mn + 50
   !if (nsym.eq.1) mn = mn + 50
   if (spacegr%symcent == 1) mn = mn + 50
   if (spacegr%nsymop  == 1) mn = mn + 50
   mn = min0(mn,499)
   nstr=mn+min0(499-mn,100)/2
400 continue
   if (active_file == REFL_FILE) then
!      read reflexion data from cards
       kuser = 3000
       call machkls(filehpow,kuser,istop,ier)
       if (ier.lt.0) go to 900
       call defovl
   endif
   if (nrefl > 0) then  
       we(:nrefl) = 1.0
       iprandomax=27
       icomatsav=0
       ipdirsav=0
       fcvet(1:nrefl) = we(1:nrefl)
       if (mpi_prog_numprocs > 1) then
           return
       endif
       call CreaListaRiflessi
!
       if (active_file == REFL_FILE) then
          if (kequirip.eq.1) call noequi
          if(kequirip.eq.1) write(lo,368)
          if(kequirip.eq.0) write(lo,369)
!
          call PreparaRiflessiOrdinati
!
       endif
       call PreparaRiflessiOrdinati
       iy=67543
       call CreaListaRiflessiOrdinati(iy)
368    format(//,'   The overall intensity for each group of ',            &
       'overlapping',/,'   reflections is equipartitioned (default)')
369    format(//,'   The overall intensity for each group of ',            &
       'overlapping',/,'   reflections is not equipartitioned      ')
   endif
   call xdump
900 continue
   return
   end subroutine data3

!-----------------------------------------------------------------------

   subroutine viewrefcell(ij)
!
!  Display reflections corrisponding to selected cell, unindexed peaks.
!  Shift is applied to peaks and pattern
!
   USE Symmetry, only: ifRhombAx
   USE Conteggi, only: theta_int,npunti
   USE indexing
   !USE Riflessi
   !USE General, only: lo,nrefl,spacegr,cell
   USE General, only: spacegr,cell,ifAutomatic,alambda
   USE VIEW
   !USE PatternRef
   USE strutil
   USE messagemod
   USE arrayutil
   USE molcom, only:jscreen
   USE spginfom
   USE variables, only: cryst,dataset,ref
   USE unit_cell
   USE datamod, only: gener
   USE peak_mod, only: pkind
   USE plotstyle
   use peak_util
   implicit none
   integer, intent(in) :: ij
   common /grafica_ntreor/ibest,cellsav(6,660),ameff(660),        &
          m20sav(660),shiftsav(660),Noindsav(660),nobssav(660),   &
          krefinc(660)
   integer :: ibest,m20sav,Noindsav,nobssav,krefinc
   real    :: cellsav,ameff,shiftsav
   real, dimension(6) :: celtemp
!corr      real, dimension(npunti) :: vet1,vet7
   character(len=80) :: line
!corr      common/sinte/ abc(3),ang(3),astar,bstar,cstar,cosas,cosbs,coscs
!corr      real :: abc,ang,astar,bstar,cstar,cosas,cosbs,coscs
!corr      common /celsav/cellas(6),porig(6)
!corr      real :: cellas,porig
   integer :: ishif,ix,iflagref
   integer, dimension(3) ::  kshif 
   integer :: ier
   integer :: i,npund
!
   celtemp(:) = cell_list(ij)%par(:)
   cell(:6) = celtemp(:)
   call cryst(1)%set_unit_cell(set_cell_type(cell))
   call frm01( (/(celtemp(i),i=1,6),(0.0,i=7,13)/) )
!
   select case (cell_list(ij)%sys)
      case ('TR')
          line= 'P -1'
      case ('MO')
          line= 'P 2/m'
      case ('HE')
          line= 'P 6/m m m'
      case ('OR')
          line= 'P m m m'
      case ('TE')
          line= 'P 4/m m m'
      case ('CU')
          line= 'P m -3 m'
   end select
   call lcase(line)
   spacegr = init_spaceg_type(line)
   call cryst(1)%set_spg(spacegr)
   call StandardSpg(line,ier)
   ishif = 0
   kshif(:) = 0
   ifRhombAx = 0
   if (rhomb_axes(spacegr%symbol_xhm,celtemp)) ifRhombAx=1
   call insym(ishif,kshif,ier)
   call yfrm02
   call sc014(ix)
   call xspinb
   call gener(dataset(1)%wave,iflagref,.false.)
   if (jscreen > 0 .and. ifAutomatic == 0) then
!
       call end_viewcellref()  ! undo previous shift
!
!      Apply shift to pattern and peaks; save shift value
       shift_ntreor = shiftsav(ij)
       shift_applied = .true.
       do i=1,npunti
             theta_int(i,7)=theta_int(i,7) - shiftsav(ij)
       enddo
       do i=1,numpeaks(pkind)
          call pkind(i)%setxd(pkind(i)%getx()-shiftsav(ij),alambda)
       enddo
!           
!      Identify unindexed lines
       select case(cell_list(ij)%pcode)
         case (NTREOR09_PROG)
           npund = min(20,numpeaks(pkind))
         case (DICVOL06_PROG)
           !npund = min(20,numpeaks(pkind))
           !npund = min(npund,indx_set(cell_list(ij)%pcode)%nlines) ! check with number of peaks from GUI
!
!          Dicvol output contains the number of indexed peaks calculated on the total number of peaks
           npund = numpeaks(pkind)
         case (MCMAILLE_PROG)
           npund = min(indx_set(MCMAILLE_PROG)%nlines,numpeaks(pkind))
       end select
       call find_unindexed(pkind,npund,ref,cell_list(ij)%nund)
!
       style(STYLE_UND_PEAKS)%vis = 1
       call vedinew(1,lsetstyle=.false.)
       call write_message('The reflections generated from cell #'//trim(i_to_s(ij))//' have been visualized.',pos=2)
       call write_message('Please check their agreement with the '//'observed pattern.',pos=3)
   endif
   end subroutine viewrefcell

