MODULE programinput
 

   implicit none

   type input_file_type
     character(len=780) :: filename = ' '
     character(len=256) :: structurename  = ' '
     character(len=780) :: datafile = ' '
     integer            :: datatype = 0
     logical            :: is_range = .false.
     real               :: minrange,maxrange 
     integer            :: kalpha = 0
     real, dimension(6) :: cell
     character(len=18)  :: spaceg
     real               :: wave = 1.540560
     integer            :: radtype = 0
     integer            :: nsyn = 0
     character(len=250) :: strcont = ' '
     character(len=250) :: frag = ' '
   end type input_file_type

   CONTAINS

   subroutine write_input_file(inp,opt,clist)
!
!  Genera file di input per expo
!
   USE fileutil
   USE filereading
   USE indexing
   type(input_file_type), intent(in)            :: inp
   type(cell_info_type), dimension(:), optional :: clist
   character(len=*), intent(in), optional       :: opt
   character(len=:), allocatable                :: sopt
   integer                                      :: j_in
   type(file_handle)                            :: finp
   integer                                      :: i
!
   call finp%fopen(inp%filename,'w')
   if (finp%good()) then
       j_in = finp%handle()

       if (present(opt)) then
           sopt = opt
       else
           sopt = ' '
       endif

       write(j_in,'(a)')'%structure '//trim(inp%structurename)
       write(j_in,'(a)')'%job '//trim(inp%structurename)
       write(j_in,'(a)')'%data'
       write(j_in,'(a)')'pattern '//trim(inp%datafile)
       select case (inp%datatype)
          case (FTYPE_COUNTS)
          case (FTYPE_XY)
          case (FTYPE_DOUBLE)
            write(j_in,'(a)')'filetype double'
          case (FTYPE_GSAS)
          case (FTYPE_CIF)
       end select
       if (inp%is_range) then
           write(j_in,'(a,f0.3,2x,f0.3)')'range ',inp%minrange,inp%maxrange
       endif
       write(j_in,'(a,f0.6)')'wave ',inp%wave
       if (inp%kalpha > 0) then
           write(j_in,'(a)')'alpha'
       endif
       if (inp%radtype == 1) write(j_in,'(a)')'neutro'
       if (inp%nsyn == 1) write(j_in,'(a)')'synchrotron'

       select case(opt)
         case ('index')
            write(j_in,'(a)')'%indexing'
            write(j_in,'(a)')'dicvol'

         case ('space')
            if (present(clist)) then
                write(j_in,'(a,3(2x,f0.6),3(2x,f0.5))')'cell',clist(1)%par
                do i=2,size(clist)
                   write(j_in,'(a,3(2x,f0.6),3(2x,f0.5))')'>cell',clist(i)%par
                enddo
            else
                write(j_in,'(a,3(2x,f0.6),3(2x,f0.5))')'cell',inp%cell(:)
            endif
            write(j_in,'(a)')'%continue'

         case ('sannel')
            write(j_in,'(a)')'space '//trim(inp%spaceg)
            write(j_in,'(a,3(2x,f0.6),3(2x,f0.5))')'cell',inp%cell(:)
            write(j_in,'(a)')'%fragment '//trim(inp%frag)
            write(j_in,'(a)')'%sannel'
            write(j_in,'(a)')'%save '//trim(inp%structurename)//'.expo'

         case default
            write(j_in,'(a)')'space '//trim(inp%spaceg)
            write(j_in,'(a,3(2x,f0.6),3(2x,f0.5))')'cell',inp%cell(:)
            write(j_in,'(a)')'cont '//trim(inp%strcont)
            write(j_in,'(a)')'%continue'

       end select

       call finp%fclose()
   endif
!
   end subroutine write_input_file

!--------------------------------------------------------------------------------------

   function input_file_create(filename) result(inp)
!
!  Estrae info per la creazione di un file di input
!
   USE General
   USE PatternRef
   USE sinterface
   USE elements 
   USE variables, only: cryst
   type(input_file_type)        :: inp
   character(len=*), intent(in) :: filename
   character(len=:), allocatable :: strfor
!
   inp%filename = filename
   inp%structurename = trim(StructureName)
   inp%datafile = filehpow
   inp%datatype = filetypepow
   inp%is_range = modify_range
   inp%minrange = minrange
   inp%maxrange = maxrange
   inp%kalpha = kalpha
   inp%cell(:) = cell(1:6)
   inp%spaceg = spaceg_string()
   inp%wave = alambda
   inp%radtype = neutro
   inp%nsyn = nsyn
   inp%strcont = cryst(1)%get_strelem()
   if (len_trim(inp%strcont) == 0) then
       call chemical_formula(celem,strfor)
       inp%strcont = strfor
   endif
!
   end function input_file_create

END MODULE programinput
